# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "ServerMonitor.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.event

import raritan.rpc.idl

import raritan.rpc.servermon


# interface
class ServerMonitor(Interface):
    idlType = "servermon.ServerMonitor:2.0.1"

    # enumeration
    class ServerReachability(Enumeration):
        idlType = "servermon.ServerMonitor_2_0_1.ServerReachability:1.0.0"
        values = ["WAITING", "REACHABLE", "UNREACHABLE", "ERROR"]

    ServerReachability.WAITING = ServerReachability(0)
    ServerReachability.REACHABLE = ServerReachability(1)
    ServerReachability.UNREACHABLE = ServerReachability(2)
    ServerReachability.ERROR = ServerReachability(3)

    # enumeration
    class ServerPowerState(Enumeration):
        idlType = "servermon.ServerMonitor_2_0_1.ServerPowerState:1.0.0"
        values = ["UNKNOWN", "ON", "OFF", "SHUTTING_DOWN"]

    ServerPowerState.UNKNOWN = ServerPowerState(0)
    ServerPowerState.ON = ServerPowerState(1)
    ServerPowerState.OFF = ServerPowerState(2)
    ServerPowerState.SHUTTING_DOWN = ServerPowerState(3)

    # enumeration
    class ServerPowerControlResult(Enumeration):
        idlType = "servermon.ServerMonitor_2_0_1.ServerPowerControlResult:1.0.0"
        values = ["NO_ERROR", "SHUTDOWN_CMD_FAILED", "SWITCHING_OFF_FAILED", "SWITCHING_ON_FAILED", "POWER_CHECK_TIMEOUT"]

    ServerPowerControlResult.NO_ERROR = ServerPowerControlResult(0)
    ServerPowerControlResult.SHUTDOWN_CMD_FAILED = ServerPowerControlResult(1)
    ServerPowerControlResult.SWITCHING_OFF_FAILED = ServerPowerControlResult(2)
    ServerPowerControlResult.SWITCHING_ON_FAILED = ServerPowerControlResult(3)
    ServerPowerControlResult.POWER_CHECK_TIMEOUT = ServerPowerControlResult(4)

    # enumeration
    class ServerPowerCheckMethod(Enumeration):
        idlType = "servermon.ServerMonitor_2_0_1.ServerPowerCheckMethod:1.0.0"
        values = ["TIMER", "POWER_DROP"]

    ServerPowerCheckMethod.TIMER = ServerPowerCheckMethod(0)
    ServerPowerCheckMethod.POWER_DROP = ServerPowerCheckMethod(1)

    # structure
    class ServerPowerSettings(Structure):
        idlType = "servermon.ServerMonitor_2_0_1.ServerPowerSettings:1.0.0"
        elements = ["enabled", "target", "powerCheck", "powerThreshold", "timeout", "shutdownCmd", "username", "password", "sshPort"]

        def __init__(self, enabled, target, powerCheck, powerThreshold, timeout, shutdownCmd, username, password, sshPort):
            typecheck.is_bool(enabled, AssertionError)
            typecheck.is_remote_obj(target, AssertionError)
            typecheck.is_enum(powerCheck, raritan.rpc.servermon.ServerMonitor.ServerPowerCheckMethod, AssertionError)
            typecheck.is_double(powerThreshold, AssertionError)
            typecheck.is_int(timeout, AssertionError)
            typecheck.is_string(shutdownCmd, AssertionError)
            typecheck.is_string(username, AssertionError)
            typecheck.is_string(password, AssertionError)
            typecheck.is_int(sshPort, AssertionError)

            self.enabled = enabled
            self.target = target
            self.powerCheck = powerCheck
            self.powerThreshold = powerThreshold
            self.timeout = timeout
            self.shutdownCmd = shutdownCmd
            self.username = username
            self.password = password
            self.sshPort = sshPort

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                enabled = json['enabled'],
                target = Interface.decode(json['target'], agent),
                powerCheck = raritan.rpc.servermon.ServerMonitor.ServerPowerCheckMethod.decode(json['powerCheck']),
                powerThreshold = json['powerThreshold'],
                timeout = json['timeout'],
                shutdownCmd = json['shutdownCmd'],
                username = json['username'],
                password = json['password'],
                sshPort = json['sshPort'],
            )
            return obj

        def encode(self):
            json = {}
            json['enabled'] = self.enabled
            json['target'] = Interface.encode(self.target)
            json['powerCheck'] = raritan.rpc.servermon.ServerMonitor.ServerPowerCheckMethod.encode(self.powerCheck)
            json['powerThreshold'] = self.powerThreshold
            json['timeout'] = self.timeout
            json['shutdownCmd'] = self.shutdownCmd
            json['username'] = self.username
            json['password'] = self.password
            json['sshPort'] = self.sshPort
            return json

    # structure
    class ServerSettings(Structure):
        idlType = "servermon.ServerMonitor_2_0_1.ServerSettings:1.0.0"
        elements = ["host", "enabled", "pingInterval", "retryInterval", "activationCount", "failureCount", "resumeDelay", "resumeCount", "powerSettings"]

        def __init__(self, host, enabled, pingInterval, retryInterval, activationCount, failureCount, resumeDelay, resumeCount, powerSettings):
            typecheck.is_string(host, AssertionError)
            typecheck.is_bool(enabled, AssertionError)
            typecheck.is_int(pingInterval, AssertionError)
            typecheck.is_int(retryInterval, AssertionError)
            typecheck.is_int(activationCount, AssertionError)
            typecheck.is_int(failureCount, AssertionError)
            typecheck.is_int(resumeDelay, AssertionError)
            typecheck.is_int(resumeCount, AssertionError)
            typecheck.is_struct(powerSettings, raritan.rpc.servermon.ServerMonitor.ServerPowerSettings, AssertionError)

            self.host = host
            self.enabled = enabled
            self.pingInterval = pingInterval
            self.retryInterval = retryInterval
            self.activationCount = activationCount
            self.failureCount = failureCount
            self.resumeDelay = resumeDelay
            self.resumeCount = resumeCount
            self.powerSettings = powerSettings

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                host = json['host'],
                enabled = json['enabled'],
                pingInterval = json['pingInterval'],
                retryInterval = json['retryInterval'],
                activationCount = json['activationCount'],
                failureCount = json['failureCount'],
                resumeDelay = json['resumeDelay'],
                resumeCount = json['resumeCount'],
                powerSettings = raritan.rpc.servermon.ServerMonitor.ServerPowerSettings.decode(json['powerSettings'], agent),
            )
            return obj

        def encode(self):
            json = {}
            json['host'] = self.host
            json['enabled'] = self.enabled
            json['pingInterval'] = self.pingInterval
            json['retryInterval'] = self.retryInterval
            json['activationCount'] = self.activationCount
            json['failureCount'] = self.failureCount
            json['resumeDelay'] = self.resumeDelay
            json['resumeCount'] = self.resumeCount
            json['powerSettings'] = raritan.rpc.servermon.ServerMonitor.ServerPowerSettings.encode(self.powerSettings)
            return json

    # structure
    class ServerStatus(Structure):
        idlType = "servermon.ServerMonitor_2_0_1.ServerStatus:1.0.0"
        elements = ["powerState", "lastPowerControlResult", "reachable", "lastRequest", "lastResponse", "requests", "responses", "failures", "resumes"]

        def __init__(self, powerState, lastPowerControlResult, reachable, lastRequest, lastResponse, requests, responses, failures, resumes):
            typecheck.is_enum(powerState, raritan.rpc.servermon.ServerMonitor.ServerPowerState, AssertionError)
            typecheck.is_enum(lastPowerControlResult, raritan.rpc.servermon.ServerMonitor.ServerPowerControlResult, AssertionError)
            typecheck.is_enum(reachable, raritan.rpc.servermon.ServerMonitor.ServerReachability, AssertionError)
            typecheck.is_time(lastRequest, AssertionError)
            typecheck.is_time(lastResponse, AssertionError)
            typecheck.is_int(requests, AssertionError)
            typecheck.is_int(responses, AssertionError)
            typecheck.is_int(failures, AssertionError)
            typecheck.is_int(resumes, AssertionError)

            self.powerState = powerState
            self.lastPowerControlResult = lastPowerControlResult
            self.reachable = reachable
            self.lastRequest = lastRequest
            self.lastResponse = lastResponse
            self.requests = requests
            self.responses = responses
            self.failures = failures
            self.resumes = resumes

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                powerState = raritan.rpc.servermon.ServerMonitor.ServerPowerState.decode(json['powerState']),
                lastPowerControlResult = raritan.rpc.servermon.ServerMonitor.ServerPowerControlResult.decode(json['lastPowerControlResult']),
                reachable = raritan.rpc.servermon.ServerMonitor.ServerReachability.decode(json['reachable']),
                lastRequest = raritan.rpc.Time.decode(json['lastRequest']),
                lastResponse = raritan.rpc.Time.decode(json['lastResponse']),
                requests = json['requests'],
                responses = json['responses'],
                failures = json['failures'],
                resumes = json['resumes'],
            )
            return obj

        def encode(self):
            json = {}
            json['powerState'] = raritan.rpc.servermon.ServerMonitor.ServerPowerState.encode(self.powerState)
            json['lastPowerControlResult'] = raritan.rpc.servermon.ServerMonitor.ServerPowerControlResult.encode(self.lastPowerControlResult)
            json['reachable'] = raritan.rpc.servermon.ServerMonitor.ServerReachability.encode(self.reachable)
            json['lastRequest'] = raritan.rpc.Time.encode(self.lastRequest)
            json['lastResponse'] = raritan.rpc.Time.encode(self.lastResponse)
            json['requests'] = self.requests
            json['responses'] = self.responses
            json['failures'] = self.failures
            json['resumes'] = self.resumes
            return json

    # value object
    class ServerPowerStateEvent(raritan.rpc.idl.Event):
        idlType = "servermon.ServerMonitor_2_0_1.ServerPowerStateEvent:1.0.0"

        def __init__(self, id, host, oldPowerState, newPowerState, source):
            super(raritan.rpc.servermon.ServerMonitor.ServerPowerStateEvent, self).__init__(source)
            typecheck.is_int(id, AssertionError)
            typecheck.is_string(host, AssertionError)
            typecheck.is_enum(oldPowerState, raritan.rpc.servermon.ServerMonitor.ServerPowerState, AssertionError)
            typecheck.is_enum(newPowerState, raritan.rpc.servermon.ServerMonitor.ServerPowerState, AssertionError)

            self.id = id
            self.host = host
            self.oldPowerState = oldPowerState
            self.newPowerState = newPowerState

        def encode(self):
            json = super(raritan.rpc.servermon.ServerMonitor.ServerPowerStateEvent, self).encode()
            json['id'] = self.id
            json['host'] = self.host
            json['oldPowerState'] = raritan.rpc.servermon.ServerMonitor.ServerPowerState.encode(self.oldPowerState)
            json['newPowerState'] = raritan.rpc.servermon.ServerMonitor.ServerPowerState.encode(self.newPowerState)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                id = json['id'],
                host = json['host'],
                oldPowerState = raritan.rpc.servermon.ServerMonitor.ServerPowerState.decode(json['oldPowerState']),
                newPowerState = raritan.rpc.servermon.ServerMonitor.ServerPowerState.decode(json['newPowerState']),
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["id", "host", "oldPowerState", "newPowerState"]
            elements = elements + super(raritan.rpc.servermon.ServerMonitor.ServerPowerStateEvent, self).listElements()
            return elements

    # value object
    class ServerPowerControlInitiatedEvent(raritan.rpc.event.UserEvent):
        idlType = "servermon.ServerMonitor_2_0_1.ServerPowerControlInitiatedEvent:1.0.0"

        def __init__(self, id, host, on, actUserName, actIpAddr, source):
            super(raritan.rpc.servermon.ServerMonitor.ServerPowerControlInitiatedEvent, self).__init__(actUserName, actIpAddr, source)
            typecheck.is_int(id, AssertionError)
            typecheck.is_string(host, AssertionError)
            typecheck.is_bool(on, AssertionError)

            self.id = id
            self.host = host
            self.on = on

        def encode(self):
            json = super(raritan.rpc.servermon.ServerMonitor.ServerPowerControlInitiatedEvent, self).encode()
            json['id'] = self.id
            json['host'] = self.host
            json['on'] = self.on
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                id = json['id'],
                host = json['host'],
                on = json['on'],
                # for event.UserEvent
                actUserName = json['actUserName'],
                actIpAddr = json['actIpAddr'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["id", "host", "on"]
            elements = elements + super(raritan.rpc.servermon.ServerMonitor.ServerPowerControlInitiatedEvent, self).listElements()
            return elements

    # value object
    class ServerPowerControlCompletedEvent(raritan.rpc.idl.Event):
        idlType = "servermon.ServerMonitor_2_0_1.ServerPowerControlCompletedEvent:1.0.0"

        def __init__(self, id, host, result, source):
            super(raritan.rpc.servermon.ServerMonitor.ServerPowerControlCompletedEvent, self).__init__(source)
            typecheck.is_int(id, AssertionError)
            typecheck.is_string(host, AssertionError)
            typecheck.is_enum(result, raritan.rpc.servermon.ServerMonitor.ServerPowerControlResult, AssertionError)

            self.id = id
            self.host = host
            self.result = result

        def encode(self):
            json = super(raritan.rpc.servermon.ServerMonitor.ServerPowerControlCompletedEvent, self).encode()
            json['id'] = self.id
            json['host'] = self.host
            json['result'] = raritan.rpc.servermon.ServerMonitor.ServerPowerControlResult.encode(self.result)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                id = json['id'],
                host = json['host'],
                result = raritan.rpc.servermon.ServerMonitor.ServerPowerControlResult.decode(json['result']),
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["id", "host", "result"]
            elements = elements + super(raritan.rpc.servermon.ServerMonitor.ServerPowerControlCompletedEvent, self).listElements()
            return elements

    # value object
    class ServerReachabilityEvent(raritan.rpc.idl.Event):
        idlType = "servermon.ServerMonitor_2_0_1.ServerReachabilityEvent:1.0.0"

        def __init__(self, id, host, reachable, source):
            super(raritan.rpc.servermon.ServerMonitor.ServerReachabilityEvent, self).__init__(source)
            typecheck.is_int(id, AssertionError)
            typecheck.is_string(host, AssertionError)
            typecheck.is_enum(reachable, raritan.rpc.servermon.ServerMonitor.ServerReachability, AssertionError)

            self.id = id
            self.host = host
            self.reachable = reachable

        def encode(self):
            json = super(raritan.rpc.servermon.ServerMonitor.ServerReachabilityEvent, self).encode()
            json['id'] = self.id
            json['host'] = self.host
            json['reachable'] = raritan.rpc.servermon.ServerMonitor.ServerReachability.encode(self.reachable)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                id = json['id'],
                host = json['host'],
                reachable = raritan.rpc.servermon.ServerMonitor.ServerReachability.decode(json['reachable']),
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["id", "host", "reachable"]
            elements = elements + super(raritan.rpc.servermon.ServerMonitor.ServerReachabilityEvent, self).listElements()
            return elements

    # value object
    class ServerAddedEvent(raritan.rpc.event.UserEvent):
        idlType = "servermon.ServerMonitor_2_0_1.ServerAddedEvent:1.0.0"

        def __init__(self, id, settings, actUserName, actIpAddr, source):
            super(raritan.rpc.servermon.ServerMonitor.ServerAddedEvent, self).__init__(actUserName, actIpAddr, source)
            typecheck.is_int(id, AssertionError)
            typecheck.is_struct(settings, raritan.rpc.servermon.ServerMonitor.ServerSettings, AssertionError)

            self.id = id
            self.settings = settings

        def encode(self):
            json = super(raritan.rpc.servermon.ServerMonitor.ServerAddedEvent, self).encode()
            json['id'] = self.id
            json['settings'] = raritan.rpc.servermon.ServerMonitor.ServerSettings.encode(self.settings)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                id = json['id'],
                settings = raritan.rpc.servermon.ServerMonitor.ServerSettings.decode(json['settings'], agent),
                # for event.UserEvent
                actUserName = json['actUserName'],
                actIpAddr = json['actIpAddr'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["id", "settings"]
            elements = elements + super(raritan.rpc.servermon.ServerMonitor.ServerAddedEvent, self).listElements()
            return elements

    # value object
    class ServerSettingsChangedEvent(raritan.rpc.event.UserEvent):
        idlType = "servermon.ServerMonitor_2_0_1.ServerSettingsChangedEvent:1.0.0"

        def __init__(self, id, oldSettings, newSettings, actUserName, actIpAddr, source):
            super(raritan.rpc.servermon.ServerMonitor.ServerSettingsChangedEvent, self).__init__(actUserName, actIpAddr, source)
            typecheck.is_int(id, AssertionError)
            typecheck.is_struct(oldSettings, raritan.rpc.servermon.ServerMonitor.ServerSettings, AssertionError)
            typecheck.is_struct(newSettings, raritan.rpc.servermon.ServerMonitor.ServerSettings, AssertionError)

            self.id = id
            self.oldSettings = oldSettings
            self.newSettings = newSettings

        def encode(self):
            json = super(raritan.rpc.servermon.ServerMonitor.ServerSettingsChangedEvent, self).encode()
            json['id'] = self.id
            json['oldSettings'] = raritan.rpc.servermon.ServerMonitor.ServerSettings.encode(self.oldSettings)
            json['newSettings'] = raritan.rpc.servermon.ServerMonitor.ServerSettings.encode(self.newSettings)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                id = json['id'],
                oldSettings = raritan.rpc.servermon.ServerMonitor.ServerSettings.decode(json['oldSettings'], agent),
                newSettings = raritan.rpc.servermon.ServerMonitor.ServerSettings.decode(json['newSettings'], agent),
                # for event.UserEvent
                actUserName = json['actUserName'],
                actIpAddr = json['actIpAddr'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["id", "oldSettings", "newSettings"]
            elements = elements + super(raritan.rpc.servermon.ServerMonitor.ServerSettingsChangedEvent, self).listElements()
            return elements

    # value object
    class ServerDeletedEvent(raritan.rpc.event.UserEvent):
        idlType = "servermon.ServerMonitor_2_0_1.ServerDeletedEvent:1.0.0"

        def __init__(self, id, actUserName, actIpAddr, source):
            super(raritan.rpc.servermon.ServerMonitor.ServerDeletedEvent, self).__init__(actUserName, actIpAddr, source)
            typecheck.is_int(id, AssertionError)

            self.id = id

        def encode(self):
            json = super(raritan.rpc.servermon.ServerMonitor.ServerDeletedEvent, self).encode()
            json['id'] = self.id
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                id = json['id'],
                # for event.UserEvent
                actUserName = json['actUserName'],
                actIpAddr = json['actIpAddr'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["id"]
            elements = elements + super(raritan.rpc.servermon.ServerMonitor.ServerDeletedEvent, self).listElements()
            return elements

    # structure
    class Server(Structure):
        idlType = "servermon.ServerMonitor_2_0_1.Server:1.0.0"
        elements = ["settings", "status"]

        def __init__(self, settings, status):
            typecheck.is_struct(settings, raritan.rpc.servermon.ServerMonitor.ServerSettings, AssertionError)
            typecheck.is_struct(status, raritan.rpc.servermon.ServerMonitor.ServerStatus, AssertionError)

            self.settings = settings
            self.status = status

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                settings = raritan.rpc.servermon.ServerMonitor.ServerSettings.decode(json['settings'], agent),
                status = raritan.rpc.servermon.ServerMonitor.ServerStatus.decode(json['status'], agent),
            )
            return obj

        def encode(self):
            json = {}
            json['settings'] = raritan.rpc.servermon.ServerMonitor.ServerSettings.encode(self.settings)
            json['status'] = raritan.rpc.servermon.ServerMonitor.ServerStatus.encode(self.status)
            return json

    ERR_NO_SUCH_ID = 1

    ERR_INVALID_SETTINGS = 2

    ERR_DUPLICATE_HOSTNAME = 3

    ERR_MAX_SERVERS_REACHED = 4

    class _addServer(Interface.Method):
        name = 'addServer'

        @staticmethod
        def encode(settings):
            typecheck.is_struct(settings, raritan.rpc.servermon.ServerMonitor.ServerSettings, AssertionError)
            args = {}
            args['settings'] = raritan.rpc.servermon.ServerMonitor.ServerSettings.encode(settings)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            id = rsp['id']
            typecheck.is_int(_ret_, DecodeException)
            typecheck.is_int(id, DecodeException)
            return (_ret_, id)

    class _modifyServer(Interface.Method):
        name = 'modifyServer'

        @staticmethod
        def encode(id, settings):
            typecheck.is_int(id, AssertionError)
            typecheck.is_struct(settings, raritan.rpc.servermon.ServerMonitor.ServerSettings, AssertionError)
            args = {}
            args['id'] = id
            args['settings'] = raritan.rpc.servermon.ServerMonitor.ServerSettings.encode(settings)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _deleteServer(Interface.Method):
        name = 'deleteServer'

        @staticmethod
        def encode(id):
            typecheck.is_int(id, AssertionError)
            args = {}
            args['id'] = id
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getServer(Interface.Method):
        name = 'getServer'

        @staticmethod
        def encode(id):
            typecheck.is_int(id, AssertionError)
            args = {}
            args['id'] = id
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            server = raritan.rpc.servermon.ServerMonitor.Server.decode(rsp['server'], agent)
            typecheck.is_int(_ret_, DecodeException)
            typecheck.is_struct(server, raritan.rpc.servermon.ServerMonitor.Server, DecodeException)
            return (_ret_, server)

    class _listServers(Interface.Method):
        name = 'listServers'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = dict([(
                elem['key'],
                raritan.rpc.servermon.ServerMonitor.Server.decode(elem['value'], agent))
                for elem in rsp['_ret_']])
            return _ret_

    class _powerControl(Interface.Method):
        name = 'powerControl'

        @staticmethod
        def encode(id, on):
            typecheck.is_int(id, AssertionError)
            typecheck.is_bool(on, AssertionError)
            args = {}
            args['id'] = id
            args['on'] = on
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(ServerMonitor, self).__init__(target, agent)
        self.addServer = ServerMonitor._addServer(self)
        self.modifyServer = ServerMonitor._modifyServer(self)
        self.deleteServer = ServerMonitor._deleteServer(self)
        self.getServer = ServerMonitor._getServer(self)
        self.listServers = ServerMonitor._listServers(self)
        self.powerControl = ServerMonitor._powerControl(self)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2010 Raritan Inc. All rights reserved.

#
# Decodes IDL "typecode" to python class and vice versa.
#

import re

# TODO: generate prefix from "base-package" in config
prefix = "raritan.rpc"

class TypeInfo(object):

    @staticmethod
    def typeBaseName(typeId):
        b = typeId.split(":")[0] # remove version
        b = re.sub(r'_[0-9]*_[0-9]*_[0-9]*', r'', b) # remove version
        return b

    @classmethod
    def idlTypeIdToPyClass(cls, typeId):
        """Returns python class for given typeId.

        The module defining this class will be auto-imported.
        """
        pyName = "%s.%s" % (prefix, TypeInfo.typeBaseName(typeId))
        comps = pyName.split(".")
        # remove components from end until import succeeds
        while comps:
            modName = ".".join(comps)
            try:
                exec("import %s" % modName)
            except ImportError:
                comps.pop()
                continue
            cls = eval(pyName)
            return cls
        raise ImportError("Unable to find package for %s." % typeId)

    @classmethod
    def pyClassToIdlName(cls, pyClass):
        return pyClass.idlType

    @classmethod
    def decode(cls, json):
        typeId = json
        return cls.idlTypeIdToPyClass(typeId)

    @classmethod
    def encode(cls, pyClass):
        return cls.pyClassToIdlName(pyClass)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2014 Raritan Inc. All rights reserved.

class JsonObject:

    @staticmethod
    def decode(json, agent):
        return json

    @staticmethod
    def encode(obj):
        return obj



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "BulkRequest.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
from raritan.rpc.opaque.bulkrpc import JsonObject

import raritan.rpc.bulkrpc


# structure
class Request(Structure):
    idlType = "bulkrpc.Request:1.0.0"
    elements = ["rid", "json"]

    def __init__(self, rid, json):
        typecheck.is_string(rid, AssertionError)
        typecheck.is_string(json, AssertionError)

        self.rid = rid
        self.json = json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            rid = json['rid'],
            json = json['json'],
        )
        return obj

    def encode(self):
        json = {}
        json['rid'] = self.rid
        json['json'] = self.json
        return json

# structure
class Response(Structure):
    idlType = "bulkrpc.Response:1.0.0"
    elements = ["json", "statcode"]

    def __init__(self, json, statcode):
        typecheck.is_string(json, AssertionError)
        typecheck.is_int(statcode, AssertionError)

        self.json = json
        self.statcode = statcode

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            json = json['json'],
            statcode = json['statcode'],
        )
        return obj

    def encode(self):
        json = {}
        json['json'] = self.json
        json['statcode'] = self.statcode
        return json

# interface
class BulkRequest(Interface):
    idlType = "bulkrpc.BulkRequest:1.0.2"

    # structure
    class Request(Structure):
        idlType = "bulkrpc.BulkRequest_1_0_2.Request:1.0.0"
        elements = ["rid", "json"]

        def __init__(self, rid, json):
            typecheck.is_string(rid, AssertionError)

            self.rid = rid
            self.json = json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                rid = json['rid'],
                json = raritan.rpc.bulkrpc.JsonObject.decode(json['json'], agent),
            )
            return obj

        def encode(self):
            json = {}
            json['rid'] = self.rid
            json['json'] = raritan.rpc.bulkrpc.JsonObject.encode(self.json)
            return json

    # structure
    class Response(Structure):
        idlType = "bulkrpc.BulkRequest_1_0_2.Response:1.0.0"
        elements = ["json", "statcode"]

        def __init__(self, json, statcode):
            typecheck.is_int(statcode, AssertionError)

            self.json = json
            self.statcode = statcode

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                json = raritan.rpc.bulkrpc.JsonObject.decode(json['json'], agent),
                statcode = json['statcode'],
            )
            return obj

        def encode(self):
            json = {}
            json['json'] = raritan.rpc.bulkrpc.JsonObject.encode(self.json)
            json['statcode'] = self.statcode
            return json

    class _performRequest(Interface.Method):
        name = 'performRequest'

        @staticmethod
        def encode(requests):
            for x0 in requests:
                typecheck.is_struct(x0, raritan.rpc.bulkrpc.Request, AssertionError)
            args = {}
            args['requests'] = [raritan.rpc.bulkrpc.Request.encode(x0) for x0 in requests]
            return args

        @staticmethod
        def decode(rsp, agent):
            responses = [raritan.rpc.bulkrpc.Response.decode(x0, agent) for x0 in rsp['responses']]
            for x0 in responses:
                typecheck.is_struct(x0, raritan.rpc.bulkrpc.Response, DecodeException)
            return responses

    class _performBulk(Interface.Method):
        name = 'performBulk'

        @staticmethod
        def encode(requests):
            for x0 in requests:
                typecheck.is_struct(x0, raritan.rpc.bulkrpc.BulkRequest.Request, AssertionError)
            args = {}
            args['requests'] = [raritan.rpc.bulkrpc.BulkRequest.Request.encode(x0) for x0 in requests]
            return args

        @staticmethod
        def decode(rsp, agent):
            responses = [raritan.rpc.bulkrpc.BulkRequest.Response.decode(x0, agent) for x0 in rsp['responses']]
            for x0 in responses:
                typecheck.is_struct(x0, raritan.rpc.bulkrpc.BulkRequest.Response, DecodeException)
            return responses

    class _performBulkTimeout(Interface.Method):
        name = 'performBulkTimeout'

        @staticmethod
        def encode(requests, timeoutMs):
            for x0 in requests:
                typecheck.is_struct(x0, raritan.rpc.bulkrpc.BulkRequest.Request, AssertionError)
            typecheck.is_int(timeoutMs, AssertionError)
            args = {}
            args['requests'] = [raritan.rpc.bulkrpc.BulkRequest.Request.encode(x0) for x0 in requests]
            args['timeoutMs'] = timeoutMs
            return args

        @staticmethod
        def decode(rsp, agent):
            responses = [raritan.rpc.bulkrpc.BulkRequest.Response.decode(x0, agent) for x0 in rsp['responses']]
            for x0 in responses:
                typecheck.is_struct(x0, raritan.rpc.bulkrpc.BulkRequest.Response, DecodeException)
            return responses
    def __init__(self, target, agent):
        super(BulkRequest, self).__init__(target, agent)
        self.performRequest = BulkRequest._performRequest(self)
        self.performBulk = BulkRequest._performBulk(self)
        self.performBulkTimeout = BulkRequest._performBulkTimeout(self)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "LhxParameter.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.idl

import raritan.rpc.lhxmodel


# interface
class Parameter(Interface):
    idlType = "lhxmodel.Parameter:2.0.1"

    ERR_INVALID_PARAMS = 1

    ERR_READ_ONLY = 2

    # enumeration
    class Unit(Enumeration):
        idlType = "lhxmodel.Parameter_2_0_1.Unit:1.0.0"
        values = ["NONE", "NUMBER", "BINARY", "TEMP_ABS", "TEMP_REL", "BAR", "PASCAL", "SIEMENS", "METER", "VOLT", "AMPERE", "HOURS", "MINUTES", "SECONDS", "TIME", "METERS_PER_SECOND", "NEWTON", "GRAMMS", "HUMIDITY_REL", "HERTZ", "OHM", "PERCENT", "LITERS_PER_MINUTE", "LITERS_PER_HOUR"]

    Unit.NONE = Unit(0)
    Unit.NUMBER = Unit(1)
    Unit.BINARY = Unit(2)
    Unit.TEMP_ABS = Unit(3)
    Unit.TEMP_REL = Unit(4)
    Unit.BAR = Unit(5)
    Unit.PASCAL = Unit(6)
    Unit.SIEMENS = Unit(7)
    Unit.METER = Unit(8)
    Unit.VOLT = Unit(9)
    Unit.AMPERE = Unit(10)
    Unit.HOURS = Unit(11)
    Unit.MINUTES = Unit(12)
    Unit.SECONDS = Unit(13)
    Unit.TIME = Unit(14)
    Unit.METERS_PER_SECOND = Unit(15)
    Unit.NEWTON = Unit(16)
    Unit.GRAMMS = Unit(17)
    Unit.HUMIDITY_REL = Unit(18)
    Unit.HERTZ = Unit(19)
    Unit.OHM = Unit(20)
    Unit.PERCENT = Unit(21)
    Unit.LITERS_PER_MINUTE = Unit(22)
    Unit.LITERS_PER_HOUR = Unit(23)

    # structure
    class Status(Structure):
        idlType = "lhxmodel.Parameter_2_0_1.Status:1.0.0"
        elements = ["switchedOn", "active", "overflow", "underflow", "valid"]

        def __init__(self, switchedOn, active, overflow, underflow, valid):
            typecheck.is_bool(switchedOn, AssertionError)
            typecheck.is_bool(active, AssertionError)
            typecheck.is_bool(overflow, AssertionError)
            typecheck.is_bool(underflow, AssertionError)
            typecheck.is_bool(valid, AssertionError)

            self.switchedOn = switchedOn
            self.active = active
            self.overflow = overflow
            self.underflow = underflow
            self.valid = valid

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                switchedOn = json['switchedOn'],
                active = json['active'],
                overflow = json['overflow'],
                underflow = json['underflow'],
                valid = json['valid'],
            )
            return obj

        def encode(self):
            json = {}
            json['switchedOn'] = self.switchedOn
            json['active'] = self.active
            json['overflow'] = self.overflow
            json['underflow'] = self.underflow
            json['valid'] = self.valid
            return json

    # structure
    class MetaData(Structure):
        idlType = "lhxmodel.Parameter_2_0_1.MetaData:1.0.0"
        elements = ["unit", "id", "defaultValue", "min", "max", "read_only", "decDigits"]

        def __init__(self, unit, id, defaultValue, min, max, read_only, decDigits):
            typecheck.is_enum(unit, raritan.rpc.lhxmodel.Parameter.Unit, AssertionError)
            typecheck.is_string(id, AssertionError)
            typecheck.is_double(defaultValue, AssertionError)
            typecheck.is_double(min, AssertionError)
            typecheck.is_double(max, AssertionError)
            typecheck.is_bool(read_only, AssertionError)
            typecheck.is_int(decDigits, AssertionError)

            self.unit = unit
            self.id = id
            self.defaultValue = defaultValue
            self.min = min
            self.max = max
            self.read_only = read_only
            self.decDigits = decDigits

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                unit = raritan.rpc.lhxmodel.Parameter.Unit.decode(json['unit']),
                id = json['id'],
                defaultValue = json['defaultValue'],
                min = json['min'],
                max = json['max'],
                read_only = json['read_only'],
                decDigits = json['decDigits'],
            )
            return obj

        def encode(self):
            json = {}
            json['unit'] = raritan.rpc.lhxmodel.Parameter.Unit.encode(self.unit)
            json['id'] = self.id
            json['defaultValue'] = self.defaultValue
            json['min'] = self.min
            json['max'] = self.max
            json['read_only'] = self.read_only
            json['decDigits'] = self.decDigits
            return json

    # structure
    class Value(Structure):
        idlType = "lhxmodel.Parameter_2_0_1.Value:1.0.0"
        elements = ["timestamp", "status", "value"]

        def __init__(self, timestamp, status, value):
            typecheck.is_time(timestamp, AssertionError)
            typecheck.is_struct(status, raritan.rpc.lhxmodel.Parameter.Status, AssertionError)
            typecheck.is_double(value, AssertionError)

            self.timestamp = timestamp
            self.status = status
            self.value = value

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                timestamp = raritan.rpc.Time.decode(json['timestamp']),
                status = raritan.rpc.lhxmodel.Parameter.Status.decode(json['status'], agent),
                value = json['value'],
            )
            return obj

        def encode(self):
            json = {}
            json['timestamp'] = raritan.rpc.Time.encode(self.timestamp)
            json['status'] = raritan.rpc.lhxmodel.Parameter.Status.encode(self.status)
            json['value'] = self.value
            return json

    # value object
    class MetaDataChangedEvent(raritan.rpc.idl.Event):
        idlType = "lhxmodel.Parameter_2_0_1.MetaDataChangedEvent:1.0.0"

        def __init__(self, oldMetaData, newMetaData, source):
            super(raritan.rpc.lhxmodel.Parameter.MetaDataChangedEvent, self).__init__(source)
            typecheck.is_struct(oldMetaData, raritan.rpc.lhxmodel.Parameter.MetaData, AssertionError)
            typecheck.is_struct(newMetaData, raritan.rpc.lhxmodel.Parameter.MetaData, AssertionError)

            self.oldMetaData = oldMetaData
            self.newMetaData = newMetaData

        def encode(self):
            json = super(raritan.rpc.lhxmodel.Parameter.MetaDataChangedEvent, self).encode()
            json['oldMetaData'] = raritan.rpc.lhxmodel.Parameter.MetaData.encode(self.oldMetaData)
            json['newMetaData'] = raritan.rpc.lhxmodel.Parameter.MetaData.encode(self.newMetaData)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                oldMetaData = raritan.rpc.lhxmodel.Parameter.MetaData.decode(json['oldMetaData'], agent),
                newMetaData = raritan.rpc.lhxmodel.Parameter.MetaData.decode(json['newMetaData'], agent),
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["oldMetaData", "newMetaData"]
            elements = elements + super(raritan.rpc.lhxmodel.Parameter.MetaDataChangedEvent, self).listElements()
            return elements

    # value object
    class ValueChangedEvent(raritan.rpc.idl.Event):
        idlType = "lhxmodel.Parameter_2_0_1.ValueChangedEvent:1.0.0"

        def __init__(self, newValue, source):
            super(raritan.rpc.lhxmodel.Parameter.ValueChangedEvent, self).__init__(source)
            typecheck.is_struct(newValue, raritan.rpc.lhxmodel.Parameter.Value, AssertionError)

            self.newValue = newValue

        def encode(self):
            json = super(raritan.rpc.lhxmodel.Parameter.ValueChangedEvent, self).encode()
            json['newValue'] = raritan.rpc.lhxmodel.Parameter.Value.encode(self.newValue)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                newValue = raritan.rpc.lhxmodel.Parameter.Value.decode(json['newValue'], agent),
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["newValue"]
            elements = elements + super(raritan.rpc.lhxmodel.Parameter.ValueChangedEvent, self).listElements()
            return elements

    class _getMetaData(Interface.Method):
        name = 'getMetaData'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.lhxmodel.Parameter.MetaData.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.lhxmodel.Parameter.MetaData, DecodeException)
            return _ret_

    class _getValue(Interface.Method):
        name = 'getValue'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.lhxmodel.Parameter.Value.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.lhxmodel.Parameter.Value, DecodeException)
            return _ret_

    class _getRawValue(Interface.Method):
        name = 'getRawValue'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _setRawValue(Interface.Method):
        name = 'setRawValue'

        @staticmethod
        def encode(rawValue, validateRange):
            typecheck.is_int(rawValue, AssertionError)
            typecheck.is_bool(validateRange, AssertionError)
            args = {}
            args['rawValue'] = rawValue
            args['validateRange'] = validateRange
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(Parameter, self).__init__(target, agent)
        self.getMetaData = Parameter._getMetaData(self)
        self.getValue = Parameter._getValue(self)
        self.getRawValue = Parameter._getRawValue(self)
        self.setRawValue = Parameter._setRawValue(self)

#
# Section generated by IdlC from "LhxSensor.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.event

import raritan.rpc.idl

import raritan.rpc.lhxmodel

import raritan.rpc.sensors


# interface
class Sensor(raritan.rpc.sensors.Sensor):
    idlType = "lhxmodel.Sensor:4.0.3"

    # structure
    class MetaData(Structure):
        idlType = "lhxmodel.Sensor_4_0_3.MetaData:1.0.0"
        elements = ["type", "numDecDigits", "numRangeMin", "numRangeMax", "numThresholdMin", "numThresholdMax", "label", "id"]

        def __init__(self, type, numDecDigits, numRangeMin, numRangeMax, numThresholdMin, numThresholdMax, label, id):
            typecheck.is_struct(type, raritan.rpc.sensors.Sensor.TypeSpec, AssertionError)
            typecheck.is_int(numDecDigits, AssertionError)
            typecheck.is_double(numRangeMin, AssertionError)
            typecheck.is_double(numRangeMax, AssertionError)
            typecheck.is_double(numThresholdMin, AssertionError)
            typecheck.is_double(numThresholdMax, AssertionError)
            typecheck.is_string(label, AssertionError)
            typecheck.is_string(id, AssertionError)

            self.type = type
            self.numDecDigits = numDecDigits
            self.numRangeMin = numRangeMin
            self.numRangeMax = numRangeMax
            self.numThresholdMin = numThresholdMin
            self.numThresholdMax = numThresholdMax
            self.label = label
            self.id = id

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                type = raritan.rpc.sensors.Sensor.TypeSpec.decode(json['type'], agent),
                numDecDigits = json['numDecDigits'],
                numRangeMin = json['numRangeMin'],
                numRangeMax = json['numRangeMax'],
                numThresholdMin = json['numThresholdMin'],
                numThresholdMax = json['numThresholdMax'],
                label = json['label'],
                id = json['id'],
            )
            return obj

        def encode(self):
            json = {}
            json['type'] = raritan.rpc.sensors.Sensor.TypeSpec.encode(self.type)
            json['numDecDigits'] = self.numDecDigits
            json['numRangeMin'] = self.numRangeMin
            json['numRangeMax'] = self.numRangeMax
            json['numThresholdMin'] = self.numThresholdMin
            json['numThresholdMax'] = self.numThresholdMax
            json['label'] = self.label
            json['id'] = self.id
            return json

    # structure
    class NumThresholds(Structure):
        idlType = "lhxmodel.Sensor_4_0_3.NumThresholds:1.0.0"
        elements = ["lowerCriticalIsEnabled", "lowerCritical", "lowerWarningIsEnabled", "lowerWarning", "upperWarningIsEnabled", "upperWarning", "upperCriticalIsEnabled", "upperCritical", "hysteresis"]

        def __init__(self, lowerCriticalIsEnabled, lowerCritical, lowerWarningIsEnabled, lowerWarning, upperWarningIsEnabled, upperWarning, upperCriticalIsEnabled, upperCritical, hysteresis):
            typecheck.is_bool(lowerCriticalIsEnabled, AssertionError)
            typecheck.is_double(lowerCritical, AssertionError)
            typecheck.is_bool(lowerWarningIsEnabled, AssertionError)
            typecheck.is_double(lowerWarning, AssertionError)
            typecheck.is_bool(upperWarningIsEnabled, AssertionError)
            typecheck.is_double(upperWarning, AssertionError)
            typecheck.is_bool(upperCriticalIsEnabled, AssertionError)
            typecheck.is_double(upperCritical, AssertionError)
            typecheck.is_double(hysteresis, AssertionError)

            self.lowerCriticalIsEnabled = lowerCriticalIsEnabled
            self.lowerCritical = lowerCritical
            self.lowerWarningIsEnabled = lowerWarningIsEnabled
            self.lowerWarning = lowerWarning
            self.upperWarningIsEnabled = upperWarningIsEnabled
            self.upperWarning = upperWarning
            self.upperCriticalIsEnabled = upperCriticalIsEnabled
            self.upperCritical = upperCritical
            self.hysteresis = hysteresis

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                lowerCriticalIsEnabled = json['lowerCriticalIsEnabled'],
                lowerCritical = json['lowerCritical'],
                lowerWarningIsEnabled = json['lowerWarningIsEnabled'],
                lowerWarning = json['lowerWarning'],
                upperWarningIsEnabled = json['upperWarningIsEnabled'],
                upperWarning = json['upperWarning'],
                upperCriticalIsEnabled = json['upperCriticalIsEnabled'],
                upperCritical = json['upperCritical'],
                hysteresis = json['hysteresis'],
            )
            return obj

        def encode(self):
            json = {}
            json['lowerCriticalIsEnabled'] = self.lowerCriticalIsEnabled
            json['lowerCritical'] = self.lowerCritical
            json['lowerWarningIsEnabled'] = self.lowerWarningIsEnabled
            json['lowerWarning'] = self.lowerWarning
            json['upperWarningIsEnabled'] = self.upperWarningIsEnabled
            json['upperWarning'] = self.upperWarning
            json['upperCriticalIsEnabled'] = self.upperCriticalIsEnabled
            json['upperCritical'] = self.upperCritical
            json['hysteresis'] = self.hysteresis
            return json

    STATE_NOT_AVAILABLE = -1

    STATE_CLOSED = 0

    STATE_OPEN = 1

    STATE_NUM_NORMAL = 0

    STATE_NUM_ABOVE_UPPER_CRITICAL = 1

    STATE_NUM_ABOVE_UPPER_WARNING = 2

    STATE_NUM_BELOW_LOWER_WARNING = 3

    STATE_NUM_BELOW_LOWER_CRITICAL = 4

    # structure
    class Reading(Structure):
        idlType = "lhxmodel.Sensor_4_0_3.Reading:1.0.0"
        elements = ["timestamp", "state", "value", "isValid"]

        def __init__(self, timestamp, state, value, isValid):
            typecheck.is_time(timestamp, AssertionError)
            typecheck.is_int(state, AssertionError)
            typecheck.is_double(value, AssertionError)
            typecheck.is_bool(isValid, AssertionError)

            self.timestamp = timestamp
            self.state = state
            self.value = value
            self.isValid = isValid

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                timestamp = raritan.rpc.Time.decode(json['timestamp']),
                state = json['state'],
                value = json['value'],
                isValid = json['isValid'],
            )
            return obj

        def encode(self):
            json = {}
            json['timestamp'] = raritan.rpc.Time.encode(self.timestamp)
            json['state'] = self.state
            json['value'] = self.value
            json['isValid'] = self.isValid
            return json

    # value object
    class ThresholdsChangedEvent(raritan.rpc.event.UserEvent):
        idlType = "lhxmodel.Sensor_4_0_3.ThresholdsChangedEvent:1.0.0"

        def __init__(self, oldThresholds, newThresholds, actUserName, actIpAddr, source):
            super(raritan.rpc.lhxmodel.Sensor.ThresholdsChangedEvent, self).__init__(actUserName, actIpAddr, source)
            typecheck.is_struct(oldThresholds, raritan.rpc.lhxmodel.Sensor.NumThresholds, AssertionError)
            typecheck.is_struct(newThresholds, raritan.rpc.lhxmodel.Sensor.NumThresholds, AssertionError)

            self.oldThresholds = oldThresholds
            self.newThresholds = newThresholds

        def encode(self):
            json = super(raritan.rpc.lhxmodel.Sensor.ThresholdsChangedEvent, self).encode()
            json['oldThresholds'] = raritan.rpc.lhxmodel.Sensor.NumThresholds.encode(self.oldThresholds)
            json['newThresholds'] = raritan.rpc.lhxmodel.Sensor.NumThresholds.encode(self.newThresholds)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                oldThresholds = raritan.rpc.lhxmodel.Sensor.NumThresholds.decode(json['oldThresholds'], agent),
                newThresholds = raritan.rpc.lhxmodel.Sensor.NumThresholds.decode(json['newThresholds'], agent),
                # for event.UserEvent
                actUserName = json['actUserName'],
                actIpAddr = json['actIpAddr'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["oldThresholds", "newThresholds"]
            elements = elements + super(raritan.rpc.lhxmodel.Sensor.ThresholdsChangedEvent, self).listElements()
            return elements

    # value object
    class StateChangedEvent(raritan.rpc.idl.Event):
        idlType = "lhxmodel.Sensor_4_0_3.StateChangedEvent:1.0.0"

        def __init__(self, oldReading, newReading, source):
            super(raritan.rpc.lhxmodel.Sensor.StateChangedEvent, self).__init__(source)
            typecheck.is_struct(oldReading, raritan.rpc.lhxmodel.Sensor.Reading, AssertionError)
            typecheck.is_struct(newReading, raritan.rpc.lhxmodel.Sensor.Reading, AssertionError)

            self.oldReading = oldReading
            self.newReading = newReading

        def encode(self):
            json = super(raritan.rpc.lhxmodel.Sensor.StateChangedEvent, self).encode()
            json['oldReading'] = raritan.rpc.lhxmodel.Sensor.Reading.encode(self.oldReading)
            json['newReading'] = raritan.rpc.lhxmodel.Sensor.Reading.encode(self.newReading)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                oldReading = raritan.rpc.lhxmodel.Sensor.Reading.decode(json['oldReading'], agent),
                newReading = raritan.rpc.lhxmodel.Sensor.Reading.decode(json['newReading'], agent),
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["oldReading", "newReading"]
            elements = elements + super(raritan.rpc.lhxmodel.Sensor.StateChangedEvent, self).listElements()
            return elements

    # value object
    class ReadingChangedEvent(raritan.rpc.idl.Event):
        idlType = "lhxmodel.Sensor_4_0_3.ReadingChangedEvent:1.0.0"

        def __init__(self, newReading, source):
            super(raritan.rpc.lhxmodel.Sensor.ReadingChangedEvent, self).__init__(source)
            typecheck.is_struct(newReading, raritan.rpc.lhxmodel.Sensor.Reading, AssertionError)

            self.newReading = newReading

        def encode(self):
            json = super(raritan.rpc.lhxmodel.Sensor.ReadingChangedEvent, self).encode()
            json['newReading'] = raritan.rpc.lhxmodel.Sensor.Reading.encode(self.newReading)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                newReading = raritan.rpc.lhxmodel.Sensor.Reading.decode(json['newReading'], agent),
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["newReading"]
            elements = elements + super(raritan.rpc.lhxmodel.Sensor.ReadingChangedEvent, self).listElements()
            return elements

    ERR_INVALID_PARAM = 1

    ERR_NOT_SUPPORTED = 2

    class _getMetaData(Interface.Method):
        name = 'getMetaData'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.lhxmodel.Sensor.MetaData.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.lhxmodel.Sensor.MetaData, DecodeException)
            return _ret_

    class _getThresholds(Interface.Method):
        name = 'getThresholds'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.lhxmodel.Sensor.NumThresholds.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.lhxmodel.Sensor.NumThresholds, DecodeException)
            return _ret_

    class _setThresholds(Interface.Method):
        name = 'setThresholds'

        @staticmethod
        def encode(thresholds):
            typecheck.is_struct(thresholds, raritan.rpc.lhxmodel.Sensor.NumThresholds, AssertionError)
            args = {}
            args['thresholds'] = raritan.rpc.lhxmodel.Sensor.NumThresholds.encode(thresholds)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getReading(Interface.Method):
        name = 'getReading'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.lhxmodel.Sensor.Reading.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.lhxmodel.Sensor.Reading, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(Sensor, self).__init__(target, agent)
        self.getMetaData = Sensor._getMetaData(self)
        self.getThresholds = Sensor._getThresholds(self)
        self.setThresholds = Sensor._setThresholds(self)
        self.getReading = Sensor._getReading(self)

#
# Section generated by IdlC from "Lhx.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.event

import raritan.rpc.idl

import raritan.rpc.lhxmodel

import raritan.rpc.sensors


# interface
class Lhx(Interface):
    idlType = "lhxmodel.Lhx:3.2.4"

    ERR_INVALID_PARAMS = 1

    ERR_NOT_SUPPORTED = 2

    # structure
    class ParamCfg(Structure):
        idlType = "lhxmodel.Lhx_3_2_4.ParamCfg:1.0.0"
        elements = ["min", "max", "decdigits"]

        def __init__(self, min, max, decdigits):
            typecheck.is_double(min, AssertionError)
            typecheck.is_double(max, AssertionError)
            typecheck.is_int(decdigits, AssertionError)

            self.min = min
            self.max = max
            self.decdigits = decdigits

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                min = json['min'],
                max = json['max'],
                decdigits = json['decdigits'],
            )
            return obj

        def encode(self):
            json = {}
            json['min'] = self.min
            json['max'] = self.max
            json['decdigits'] = self.decdigits
            return json

    # structure
    class MetaData(Structure):
        idlType = "lhxmodel.Lhx_3_2_4.MetaData:1.0.0"
        elements = ["model", "version", "setpointWaterValveCfg", "setpointVentilatorsCfg", "defaultFanSpeedCfg"]

        def __init__(self, model, version, setpointWaterValveCfg, setpointVentilatorsCfg, defaultFanSpeedCfg):
            typecheck.is_string(model, AssertionError)
            typecheck.is_string(version, AssertionError)
            typecheck.is_struct(setpointWaterValveCfg, raritan.rpc.lhxmodel.Lhx.ParamCfg, AssertionError)
            typecheck.is_struct(setpointVentilatorsCfg, raritan.rpc.lhxmodel.Lhx.ParamCfg, AssertionError)
            typecheck.is_struct(defaultFanSpeedCfg, raritan.rpc.lhxmodel.Lhx.ParamCfg, AssertionError)

            self.model = model
            self.version = version
            self.setpointWaterValveCfg = setpointWaterValveCfg
            self.setpointVentilatorsCfg = setpointVentilatorsCfg
            self.defaultFanSpeedCfg = defaultFanSpeedCfg

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                model = json['model'],
                version = json['version'],
                setpointWaterValveCfg = raritan.rpc.lhxmodel.Lhx.ParamCfg.decode(json['setpointWaterValveCfg'], agent),
                setpointVentilatorsCfg = raritan.rpc.lhxmodel.Lhx.ParamCfg.decode(json['setpointVentilatorsCfg'], agent),
                defaultFanSpeedCfg = raritan.rpc.lhxmodel.Lhx.ParamCfg.decode(json['defaultFanSpeedCfg'], agent),
            )
            return obj

        def encode(self):
            json = {}
            json['model'] = self.model
            json['version'] = self.version
            json['setpointWaterValveCfg'] = raritan.rpc.lhxmodel.Lhx.ParamCfg.encode(self.setpointWaterValveCfg)
            json['setpointVentilatorsCfg'] = raritan.rpc.lhxmodel.Lhx.ParamCfg.encode(self.setpointVentilatorsCfg)
            json['defaultFanSpeedCfg'] = raritan.rpc.lhxmodel.Lhx.ParamCfg.encode(self.defaultFanSpeedCfg)
            return json

    # structure
    class Settings(Structure):
        idlType = "lhxmodel.Lhx_3_2_4.Settings:1.0.0"
        elements = ["setpointWaterValve", "setpointVentilators", "defaultFanSpeed"]

        def __init__(self, setpointWaterValve, setpointVentilators, defaultFanSpeed):
            typecheck.is_double(setpointWaterValve, AssertionError)
            typecheck.is_double(setpointVentilators, AssertionError)
            typecheck.is_double(defaultFanSpeed, AssertionError)

            self.setpointWaterValve = setpointWaterValve
            self.setpointVentilators = setpointVentilators
            self.defaultFanSpeed = defaultFanSpeed

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                setpointWaterValve = json['setpointWaterValve'],
                setpointVentilators = json['setpointVentilators'],
                defaultFanSpeed = json['defaultFanSpeed'],
            )
            return obj

        def encode(self):
            json = {}
            json['setpointWaterValve'] = self.setpointWaterValve
            json['setpointVentilators'] = self.setpointVentilators
            json['defaultFanSpeed'] = self.defaultFanSpeed
            return json

    # structure
    class AlertStatus(Structure):
        idlType = "lhxmodel.Lhx_3_2_4.AlertStatus:1.0.0"
        elements = ["sensorFailure", "fanFailure", "powerSupplyFailure", "thresholdAirOutlet", "thresholdAirInlet", "thresholdWaterInlet", "doorOpened", "maximumCoolingRequest", "emergencyCooling", "waterLeak", "thresholdHumidity", "externalWaterCoolingFailure", "thresholdWaterOutlet", "stBusError", "condenserPumpFailure", "baseElectronicsFailure", "voltageLow"]

        def __init__(self, sensorFailure, fanFailure, powerSupplyFailure, thresholdAirOutlet, thresholdAirInlet, thresholdWaterInlet, doorOpened, maximumCoolingRequest, emergencyCooling, waterLeak, thresholdHumidity, externalWaterCoolingFailure, thresholdWaterOutlet, stBusError, condenserPumpFailure, baseElectronicsFailure, voltageLow):
            for x0 in sensorFailure:
                typecheck.is_bool(x0, AssertionError)
            for x0 in fanFailure:
                typecheck.is_bool(x0, AssertionError)
            for x0 in powerSupplyFailure:
                typecheck.is_bool(x0, AssertionError)
            typecheck.is_bool(thresholdAirOutlet, AssertionError)
            typecheck.is_bool(thresholdAirInlet, AssertionError)
            typecheck.is_bool(thresholdWaterInlet, AssertionError)
            typecheck.is_bool(doorOpened, AssertionError)
            typecheck.is_bool(maximumCoolingRequest, AssertionError)
            typecheck.is_bool(emergencyCooling, AssertionError)
            typecheck.is_bool(waterLeak, AssertionError)
            typecheck.is_bool(thresholdHumidity, AssertionError)
            typecheck.is_bool(externalWaterCoolingFailure, AssertionError)
            typecheck.is_bool(thresholdWaterOutlet, AssertionError)
            typecheck.is_bool(stBusError, AssertionError)
            typecheck.is_bool(condenserPumpFailure, AssertionError)
            typecheck.is_bool(baseElectronicsFailure, AssertionError)
            typecheck.is_bool(voltageLow, AssertionError)

            self.sensorFailure = sensorFailure
            self.fanFailure = fanFailure
            self.powerSupplyFailure = powerSupplyFailure
            self.thresholdAirOutlet = thresholdAirOutlet
            self.thresholdAirInlet = thresholdAirInlet
            self.thresholdWaterInlet = thresholdWaterInlet
            self.doorOpened = doorOpened
            self.maximumCoolingRequest = maximumCoolingRequest
            self.emergencyCooling = emergencyCooling
            self.waterLeak = waterLeak
            self.thresholdHumidity = thresholdHumidity
            self.externalWaterCoolingFailure = externalWaterCoolingFailure
            self.thresholdWaterOutlet = thresholdWaterOutlet
            self.stBusError = stBusError
            self.condenserPumpFailure = condenserPumpFailure
            self.baseElectronicsFailure = baseElectronicsFailure
            self.voltageLow = voltageLow

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                sensorFailure = [x0 for x0 in json['sensorFailure']],
                fanFailure = [x0 for x0 in json['fanFailure']],
                powerSupplyFailure = [x0 for x0 in json['powerSupplyFailure']],
                thresholdAirOutlet = json['thresholdAirOutlet'],
                thresholdAirInlet = json['thresholdAirInlet'],
                thresholdWaterInlet = json['thresholdWaterInlet'],
                doorOpened = json['doorOpened'],
                maximumCoolingRequest = json['maximumCoolingRequest'],
                emergencyCooling = json['emergencyCooling'],
                waterLeak = json['waterLeak'],
                thresholdHumidity = json['thresholdHumidity'],
                externalWaterCoolingFailure = json['externalWaterCoolingFailure'],
                thresholdWaterOutlet = json['thresholdWaterOutlet'],
                stBusError = json['stBusError'],
                condenserPumpFailure = json['condenserPumpFailure'],
                baseElectronicsFailure = json['baseElectronicsFailure'],
                voltageLow = json['voltageLow'],
            )
            return obj

        def encode(self):
            json = {}
            json['sensorFailure'] = [x0 for x0 in self.sensorFailure]
            json['fanFailure'] = [x0 for x0 in self.fanFailure]
            json['powerSupplyFailure'] = [x0 for x0 in self.powerSupplyFailure]
            json['thresholdAirOutlet'] = self.thresholdAirOutlet
            json['thresholdAirInlet'] = self.thresholdAirInlet
            json['thresholdWaterInlet'] = self.thresholdWaterInlet
            json['doorOpened'] = self.doorOpened
            json['maximumCoolingRequest'] = self.maximumCoolingRequest
            json['emergencyCooling'] = self.emergencyCooling
            json['waterLeak'] = self.waterLeak
            json['thresholdHumidity'] = self.thresholdHumidity
            json['externalWaterCoolingFailure'] = self.externalWaterCoolingFailure
            json['thresholdWaterOutlet'] = self.thresholdWaterOutlet
            json['stBusError'] = self.stBusError
            json['condenserPumpFailure'] = self.condenserPumpFailure
            json['baseElectronicsFailure'] = self.baseElectronicsFailure
            json['voltageLow'] = self.voltageLow
            return json

    # structure
    class OpState(Structure):
        idlType = "lhxmodel.Lhx_3_2_4.OpState:1.0.0"
        elements = ["on", "alertStatus", "operatingHoursLhx", "operatingHoursFan"]

        def __init__(self, on, alertStatus, operatingHoursLhx, operatingHoursFan):
            typecheck.is_bool(on, AssertionError)
            typecheck.is_struct(alertStatus, raritan.rpc.lhxmodel.Lhx.AlertStatus, AssertionError)
            typecheck.is_int(operatingHoursLhx, AssertionError)
            for x0 in operatingHoursFan:
                typecheck.is_int(x0, AssertionError)

            self.on = on
            self.alertStatus = alertStatus
            self.operatingHoursLhx = operatingHoursLhx
            self.operatingHoursFan = operatingHoursFan

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                on = json['on'],
                alertStatus = raritan.rpc.lhxmodel.Lhx.AlertStatus.decode(json['alertStatus'], agent),
                operatingHoursLhx = json['operatingHoursLhx'],
                operatingHoursFan = [x0 for x0 in json['operatingHoursFan']],
            )
            return obj

        def encode(self):
            json = {}
            json['on'] = self.on
            json['alertStatus'] = raritan.rpc.lhxmodel.Lhx.AlertStatus.encode(self.alertStatus)
            json['operatingHoursLhx'] = self.operatingHoursLhx
            json['operatingHoursFan'] = [x0 for x0 in self.operatingHoursFan]
            return json

    # structure
    class Capabilities(Structure):
        idlType = "lhxmodel.Lhx_3_2_4.Capabilities:1.0.0"
        elements = ["alerts", "features"]

        def __init__(self, alerts, features):
            typecheck.is_struct(alerts, raritan.rpc.lhxmodel.Lhx.AlertStatus, AssertionError)

            self.alerts = alerts
            self.features = features

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                alerts = raritan.rpc.lhxmodel.Lhx.AlertStatus.decode(json['alerts'], agent),
                features = dict([(
                    elem['key'],
                    elem['value'])
                    for elem in json['features']]),
            )
            return obj

        def encode(self):
            json = {}
            json['alerts'] = raritan.rpc.lhxmodel.Lhx.AlertStatus.encode(self.alerts)
            json['features'] = [dict(
                key = k,
                value = v)
                for k, v in self.features.items()]
            return json

    # value object
    class OpStateChangedEvent(raritan.rpc.idl.Event):
        idlType = "lhxmodel.Lhx_3_2_4.OpStateChangedEvent:1.0.0"

        def __init__(self, oldState, newState, source):
            super(raritan.rpc.lhxmodel.Lhx.OpStateChangedEvent, self).__init__(source)
            typecheck.is_struct(oldState, raritan.rpc.lhxmodel.Lhx.OpState, AssertionError)
            typecheck.is_struct(newState, raritan.rpc.lhxmodel.Lhx.OpState, AssertionError)

            self.oldState = oldState
            self.newState = newState

        def encode(self):
            json = super(raritan.rpc.lhxmodel.Lhx.OpStateChangedEvent, self).encode()
            json['oldState'] = raritan.rpc.lhxmodel.Lhx.OpState.encode(self.oldState)
            json['newState'] = raritan.rpc.lhxmodel.Lhx.OpState.encode(self.newState)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                oldState = raritan.rpc.lhxmodel.Lhx.OpState.decode(json['oldState'], agent),
                newState = raritan.rpc.lhxmodel.Lhx.OpState.decode(json['newState'], agent),
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["oldState", "newState"]
            elements = elements + super(raritan.rpc.lhxmodel.Lhx.OpStateChangedEvent, self).listElements()
            return elements

    # value object
    class SettingsChangedEvent(raritan.rpc.event.UserEvent):
        idlType = "lhxmodel.Lhx_3_2_4.SettingsChangedEvent:1.0.0"

        def __init__(self, oldSettings, newSettings, actUserName, actIpAddr, source):
            super(raritan.rpc.lhxmodel.Lhx.SettingsChangedEvent, self).__init__(actUserName, actIpAddr, source)
            typecheck.is_struct(oldSettings, raritan.rpc.lhxmodel.Lhx.Settings, AssertionError)
            typecheck.is_struct(newSettings, raritan.rpc.lhxmodel.Lhx.Settings, AssertionError)

            self.oldSettings = oldSettings
            self.newSettings = newSettings

        def encode(self):
            json = super(raritan.rpc.lhxmodel.Lhx.SettingsChangedEvent, self).encode()
            json['oldSettings'] = raritan.rpc.lhxmodel.Lhx.Settings.encode(self.oldSettings)
            json['newSettings'] = raritan.rpc.lhxmodel.Lhx.Settings.encode(self.newSettings)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                oldSettings = raritan.rpc.lhxmodel.Lhx.Settings.decode(json['oldSettings'], agent),
                newSettings = raritan.rpc.lhxmodel.Lhx.Settings.decode(json['newSettings'], agent),
                # for event.UserEvent
                actUserName = json['actUserName'],
                actIpAddr = json['actIpAddr'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["oldSettings", "newSettings"]
            elements = elements + super(raritan.rpc.lhxmodel.Lhx.SettingsChangedEvent, self).listElements()
            return elements

    class _getCapabilities(Interface.Method):
        name = 'getCapabilities'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.lhxmodel.Lhx.Capabilities.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.lhxmodel.Lhx.Capabilities, DecodeException)
            return _ret_

    class _getMetaData(Interface.Method):
        name = 'getMetaData'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.lhxmodel.Lhx.MetaData.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.lhxmodel.Lhx.MetaData, DecodeException)
            return _ret_

    class _getSettings(Interface.Method):
        name = 'getSettings'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.lhxmodel.Lhx.Settings.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.lhxmodel.Lhx.Settings, DecodeException)
            return _ret_

    class _setSettings(Interface.Method):
        name = 'setSettings'

        @staticmethod
        def encode(settings):
            typecheck.is_struct(settings, raritan.rpc.lhxmodel.Lhx.Settings, AssertionError)
            args = {}
            args['settings'] = raritan.rpc.lhxmodel.Lhx.Settings.encode(settings)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getSensors(Interface.Method):
        name = 'getSensors'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [Interface.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_interface(x0, raritan.rpc.lhxmodel.Sensor, DecodeException)
            return _ret_

    class _getOpState(Interface.Method):
        name = 'getOpState'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.lhxmodel.Lhx.OpState.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.lhxmodel.Lhx.OpState, DecodeException)
            return _ret_

    class _setPowerState(Interface.Method):
        name = 'setPowerState'

        @staticmethod
        def encode(state):
            typecheck.is_enum(state, raritan.rpc.sensors.Sensor.OnOffState, AssertionError)
            args = {}
            args['state'] = raritan.rpc.sensors.Sensor.OnOffState.encode(state)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getParameters(Interface.Method):
        name = 'getParameters'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [Interface.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_interface(x0, raritan.rpc.lhxmodel.Parameter, DecodeException)
            return _ret_

    class _getActualValues(Interface.Method):
        name = 'getActualValues'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [Interface.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_interface(x0, raritan.rpc.lhxmodel.Parameter, DecodeException)
            return _ret_

    class _setMaximumCoolingRequest(Interface.Method):
        name = 'setMaximumCoolingRequest'

        @staticmethod
        def encode(requested):
            typecheck.is_bool(requested, AssertionError)
            args = {}
            args['requested'] = requested
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _acknowledgeAlertStatus(Interface.Method):
        name = 'acknowledgeAlertStatus'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(Lhx, self).__init__(target, agent)
        self.getCapabilities = Lhx._getCapabilities(self)
        self.getMetaData = Lhx._getMetaData(self)
        self.getSettings = Lhx._getSettings(self)
        self.setSettings = Lhx._setSettings(self)
        self.getSensors = Lhx._getSensors(self)
        self.getOpState = Lhx._getOpState(self)
        self.setPowerState = Lhx._setPowerState(self)
        self.getParameters = Lhx._getParameters(self)
        self.getActualValues = Lhx._getActualValues(self)
        self.setMaximumCoolingRequest = Lhx._setMaximumCoolingRequest(self)
        self.acknowledgeAlertStatus = Lhx._acknowledgeAlertStatus(self)

#
# Section generated by IdlC from "LhxDevel.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.lhxmodel


# interface
class LhxDevel(Lhx):
    idlType = "lhxmodel.LhxDevel:1.1.2"

    # structure
    class ParameterData(Structure):
        idlType = "lhxmodel.LhxDevel_1_1_2.ParameterData:1.0.0"
        elements = ["value", "rawValue", "metadata"]

        def __init__(self, value, rawValue, metadata):
            typecheck.is_double(value, AssertionError)
            typecheck.is_int(rawValue, AssertionError)
            typecheck.is_struct(metadata, raritan.rpc.lhxmodel.Parameter.MetaData, AssertionError)

            self.value = value
            self.rawValue = rawValue
            self.metadata = metadata

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                value = json['value'],
                rawValue = json['rawValue'],
                metadata = raritan.rpc.lhxmodel.Parameter.MetaData.decode(json['metadata'], agent),
            )
            return obj

        def encode(self):
            json = {}
            json['value'] = self.value
            json['rawValue'] = self.rawValue
            json['metadata'] = raritan.rpc.lhxmodel.Parameter.MetaData.encode(self.metadata)
            return json

    class _dumpParameter(Interface.Method):
        name = 'dumpParameter'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.lhxmodel.LhxDevel.ParameterData.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.lhxmodel.LhxDevel.ParameterData, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(LhxDevel, self).__init__(target, agent)
        self.dumpParameter = LhxDevel._dumpParameter(self)

#
# Section generated by IdlC from "LhxConfig.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.event

import raritan.rpc.lhxmodel


# interface
class Config(Interface):
    idlType = "lhxmodel.Config:1.0.1"

    NO_ERROR = 0

    ERR_INVALID_PARAMS = 1

    # structure
    class ComSettings(Structure):
        idlType = "lhxmodel.Config_1_0_1.ComSettings:1.0.0"
        elements = ["devAddr", "lhxAddr"]

        def __init__(self, devAddr, lhxAddr):
            typecheck.is_byte(devAddr, AssertionError)
            typecheck.is_byte(lhxAddr, AssertionError)

            self.devAddr = devAddr
            self.lhxAddr = lhxAddr

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                devAddr = json['devAddr'],
                lhxAddr = json['lhxAddr'],
            )
            return obj

        def encode(self):
            json = {}
            json['devAddr'] = self.devAddr
            json['lhxAddr'] = self.lhxAddr
            return json

    # value object
    class ComSettingsChangedEvent(raritan.rpc.event.UserEvent):
        idlType = "lhxmodel.Config_1_0_1.ComSettingsChangedEvent:1.0.0"

        def __init__(self, oldSettings, newSettings, actUserName, actIpAddr, source):
            super(raritan.rpc.lhxmodel.Config.ComSettingsChangedEvent, self).__init__(actUserName, actIpAddr, source)
            typecheck.is_struct(oldSettings, raritan.rpc.lhxmodel.Config.ComSettings, AssertionError)
            typecheck.is_struct(newSettings, raritan.rpc.lhxmodel.Config.ComSettings, AssertionError)

            self.oldSettings = oldSettings
            self.newSettings = newSettings

        def encode(self):
            json = super(raritan.rpc.lhxmodel.Config.ComSettingsChangedEvent, self).encode()
            json['oldSettings'] = raritan.rpc.lhxmodel.Config.ComSettings.encode(self.oldSettings)
            json['newSettings'] = raritan.rpc.lhxmodel.Config.ComSettings.encode(self.newSettings)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                oldSettings = raritan.rpc.lhxmodel.Config.ComSettings.decode(json['oldSettings'], agent),
                newSettings = raritan.rpc.lhxmodel.Config.ComSettings.decode(json['newSettings'], agent),
                # for event.UserEvent
                actUserName = json['actUserName'],
                actIpAddr = json['actIpAddr'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["oldSettings", "newSettings"]
            elements = elements + super(raritan.rpc.lhxmodel.Config.ComSettingsChangedEvent, self).listElements()
            return elements

    # value object
    class PortNameChangedEvent(raritan.rpc.event.UserEvent):
        idlType = "lhxmodel.Config_1_0_1.PortNameChangedEvent:1.0.0"

        def __init__(self, oldName, newName, actUserName, actIpAddr, source):
            super(raritan.rpc.lhxmodel.Config.PortNameChangedEvent, self).__init__(actUserName, actIpAddr, source)
            typecheck.is_string(oldName, AssertionError)
            typecheck.is_string(newName, AssertionError)

            self.oldName = oldName
            self.newName = newName

        def encode(self):
            json = super(raritan.rpc.lhxmodel.Config.PortNameChangedEvent, self).encode()
            json['oldName'] = self.oldName
            json['newName'] = self.newName
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                oldName = json['oldName'],
                newName = json['newName'],
                # for event.UserEvent
                actUserName = json['actUserName'],
                actIpAddr = json['actIpAddr'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["oldName", "newName"]
            elements = elements + super(raritan.rpc.lhxmodel.Config.PortNameChangedEvent, self).listElements()
            return elements

    class _getComSettings(Interface.Method):
        name = 'getComSettings'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.lhxmodel.Config.ComSettings.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.lhxmodel.Config.ComSettings, DecodeException)
            return _ret_

    class _setComSettings(Interface.Method):
        name = 'setComSettings'

        @staticmethod
        def encode(settings):
            typecheck.is_struct(settings, raritan.rpc.lhxmodel.Config.ComSettings, AssertionError)
            args = {}
            args['settings'] = raritan.rpc.lhxmodel.Config.ComSettings.encode(settings)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getName(Interface.Method):
        name = 'getName'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_string(_ret_, DecodeException)
            return _ret_

    class _setName(Interface.Method):
        name = 'setName'

        @staticmethod
        def encode(name):
            typecheck.is_string(name, AssertionError)
            args = {}
            args['name'] = name
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(Config, self).__init__(target, agent)
        self.getComSettings = Config._getComSettings(self)
        self.setComSettings = Config._setComSettings(self)
        self.getName = Config._getName(self)
        self.setName = Config._setName(self)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "PeripheralDeviceSlot.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.event

import raritan.rpc.idl

import raritan.rpc.peripheral

import raritan.rpc.sensors


# enumeration
class PortType(Enumeration):
    idlType = "peripheral.PortType:1.0.0"
    values = ["ONEWIRE_ONBOARD", "ONEWIRE_DEV_PORT", "ONEWIRE_HUB_PORT", "ONEWIRE_CHAIN_POS"]

PortType.ONEWIRE_ONBOARD = PortType(0)
PortType.ONEWIRE_DEV_PORT = PortType(1)
PortType.ONEWIRE_HUB_PORT = PortType(2)
PortType.ONEWIRE_CHAIN_POS = PortType(3)

# structure
class PosElement(Structure):
    idlType = "peripheral.PosElement:1.0.0"
    elements = ["portType", "port"]

    def __init__(self, portType, port):
        typecheck.is_enum(portType, raritan.rpc.peripheral.PortType, AssertionError)
        typecheck.is_string(port, AssertionError)

        self.portType = portType
        self.port = port

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            portType = raritan.rpc.peripheral.PortType.decode(json['portType']),
            port = json['port'],
        )
        return obj

    def encode(self):
        json = {}
        json['portType'] = raritan.rpc.peripheral.PortType.encode(self.portType)
        json['port'] = self.port
        return json

# structure
class DeviceID(Structure):
    idlType = "peripheral.DeviceID:4.0.0"
    elements = ["serial", "type", "isActuator", "channel"]

    def __init__(self, serial, type, isActuator, channel):
        typecheck.is_string(serial, AssertionError)
        typecheck.is_struct(type, raritan.rpc.sensors.Sensor.TypeSpec, AssertionError)
        typecheck.is_bool(isActuator, AssertionError)
        typecheck.is_int(channel, AssertionError)

        self.serial = serial
        self.type = type
        self.isActuator = isActuator
        self.channel = channel

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            serial = json['serial'],
            type = raritan.rpc.sensors.Sensor.TypeSpec.decode(json['type'], agent),
            isActuator = json['isActuator'],
            channel = json['channel'],
        )
        return obj

    def encode(self):
        json = {}
        json['serial'] = self.serial
        json['type'] = raritan.rpc.sensors.Sensor.TypeSpec.encode(self.type)
        json['isActuator'] = self.isActuator
        json['channel'] = self.channel
        return json

# structure
class Address(Structure):
    idlType = "peripheral.Address:4.0.0"
    elements = ["position", "type", "isActuator", "channel"]

    def __init__(self, position, type, isActuator, channel):
        for x0 in position:
            typecheck.is_struct(x0, raritan.rpc.peripheral.PosElement, AssertionError)
        typecheck.is_struct(type, raritan.rpc.sensors.Sensor.TypeSpec, AssertionError)
        typecheck.is_bool(isActuator, AssertionError)
        typecheck.is_int(channel, AssertionError)

        self.position = position
        self.type = type
        self.isActuator = isActuator
        self.channel = channel

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            position = [raritan.rpc.peripheral.PosElement.decode(x0, agent) for x0 in json['position']],
            type = raritan.rpc.sensors.Sensor.TypeSpec.decode(json['type'], agent),
            isActuator = json['isActuator'],
            channel = json['channel'],
        )
        return obj

    def encode(self):
        json = {}
        json['position'] = [raritan.rpc.peripheral.PosElement.encode(x0) for x0 in self.position]
        json['type'] = raritan.rpc.sensors.Sensor.TypeSpec.encode(self.type)
        json['isActuator'] = self.isActuator
        json['channel'] = self.channel
        return json

# value object
class Device(ValueObject):
    idlType = "peripheral.Device:3.0.2"

    def __init__(self, deviceID, position, packageClass, device):
        typecheck.is_struct(deviceID, raritan.rpc.peripheral.DeviceID, AssertionError)
        for x0 in position:
            typecheck.is_struct(x0, raritan.rpc.peripheral.PosElement, AssertionError)
        typecheck.is_string(packageClass, AssertionError)
        typecheck.is_interface(device, raritan.rpc.sensors.Sensor, AssertionError)

        self.deviceID = deviceID
        self.position = position
        self.packageClass = packageClass
        self.device = device

    def encode(self):
        json = {}
        json['deviceID'] = raritan.rpc.peripheral.DeviceID.encode(self.deviceID)
        json['position'] = [raritan.rpc.peripheral.PosElement.encode(x0) for x0 in self.position]
        json['packageClass'] = self.packageClass
        json['device'] = Interface.encode(self.device)
        return json

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            deviceID = raritan.rpc.peripheral.DeviceID.decode(json['deviceID'], agent),
            position = [raritan.rpc.peripheral.PosElement.decode(x0, agent) for x0 in json['position']],
            packageClass = json['packageClass'],
            device = Interface.decode(json['device'], agent),
        )
        return obj

    def listElements(self):
        elements = ["deviceID", "position", "packageClass", "device"]
        return elements

# interface
class DeviceSlot(Interface):
    idlType = "peripheral.DeviceSlot:2.0.3"

    ERR_INVALID_PARAMS = 1

    ERR_NOT_SUPPORTED = 2

    CHANNEL_INVALID = -1

    # structure
    class Location(Structure):
        idlType = "peripheral.DeviceSlot_2_0_3.Location:1.0.0"
        elements = ["x", "y", "z"]

        def __init__(self, x, y, z):
            typecheck.is_string(x, AssertionError)
            typecheck.is_string(y, AssertionError)
            typecheck.is_string(z, AssertionError)

            self.x = x
            self.y = y
            self.z = z

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                x = json['x'],
                y = json['y'],
                z = json['z'],
            )
            return obj

        def encode(self):
            json = {}
            json['x'] = self.x
            json['y'] = self.y
            json['z'] = self.z
            return json

    # structure
    class Settings(Structure):
        idlType = "peripheral.DeviceSlot_2_0_3.Settings:1.0.0"
        elements = ["name", "description", "location", "useDefaultThresholds", "properties"]

        def __init__(self, name, description, location, useDefaultThresholds, properties):
            typecheck.is_string(name, AssertionError)
            typecheck.is_string(description, AssertionError)
            typecheck.is_struct(location, raritan.rpc.peripheral.DeviceSlot.Location, AssertionError)
            typecheck.is_bool(useDefaultThresholds, AssertionError)

            self.name = name
            self.description = description
            self.location = location
            self.useDefaultThresholds = useDefaultThresholds
            self.properties = properties

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                name = json['name'],
                description = json['description'],
                location = raritan.rpc.peripheral.DeviceSlot.Location.decode(json['location'], agent),
                useDefaultThresholds = json['useDefaultThresholds'],
                properties = dict([(
                    elem['key'],
                    elem['value'])
                    for elem in json['properties']]),
            )
            return obj

        def encode(self):
            json = {}
            json['name'] = self.name
            json['description'] = self.description
            json['location'] = raritan.rpc.peripheral.DeviceSlot.Location.encode(self.location)
            json['useDefaultThresholds'] = self.useDefaultThresholds
            json['properties'] = [dict(
                key = k,
                value = v)
                for k, v in self.properties.items()]
            return json

    # value object
    class DeviceChangedEvent(raritan.rpc.idl.Event):
        idlType = "peripheral.DeviceSlot_2_0_3.DeviceChangedEvent:1.0.0"

        def __init__(self, oldDevice, newDevice, source):
            super(raritan.rpc.peripheral.DeviceSlot.DeviceChangedEvent, self).__init__(source)
            typecheck.is_valobj(oldDevice, raritan.rpc.peripheral.Device, AssertionError)
            typecheck.is_valobj(newDevice, raritan.rpc.peripheral.Device, AssertionError)

            self.oldDevice = oldDevice
            self.newDevice = newDevice

        def encode(self):
            json = super(raritan.rpc.peripheral.DeviceSlot.DeviceChangedEvent, self).encode()
            json['oldDevice'] = ValueObject.encode(self.oldDevice)
            json['newDevice'] = ValueObject.encode(self.newDevice)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                oldDevice = ValueObject.decode(json['oldDevice'], agent),
                newDevice = ValueObject.decode(json['newDevice'], agent),
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["oldDevice", "newDevice"]
            elements = elements + super(raritan.rpc.peripheral.DeviceSlot.DeviceChangedEvent, self).listElements()
            return elements

    # value object
    class SettingsChangedEvent(raritan.rpc.event.UserEvent):
        idlType = "peripheral.DeviceSlot_2_0_3.SettingsChangedEvent:1.0.0"

        def __init__(self, oldSettings, newSettings, actUserName, actIpAddr, source):
            super(raritan.rpc.peripheral.DeviceSlot.SettingsChangedEvent, self).__init__(actUserName, actIpAddr, source)
            typecheck.is_struct(oldSettings, raritan.rpc.peripheral.DeviceSlot.Settings, AssertionError)
            typecheck.is_struct(newSettings, raritan.rpc.peripheral.DeviceSlot.Settings, AssertionError)

            self.oldSettings = oldSettings
            self.newSettings = newSettings

        def encode(self):
            json = super(raritan.rpc.peripheral.DeviceSlot.SettingsChangedEvent, self).encode()
            json['oldSettings'] = raritan.rpc.peripheral.DeviceSlot.Settings.encode(self.oldSettings)
            json['newSettings'] = raritan.rpc.peripheral.DeviceSlot.Settings.encode(self.newSettings)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                oldSettings = raritan.rpc.peripheral.DeviceSlot.Settings.decode(json['oldSettings'], agent),
                newSettings = raritan.rpc.peripheral.DeviceSlot.Settings.decode(json['newSettings'], agent),
                # for event.UserEvent
                actUserName = json['actUserName'],
                actIpAddr = json['actIpAddr'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["oldSettings", "newSettings"]
            elements = elements + super(raritan.rpc.peripheral.DeviceSlot.SettingsChangedEvent, self).listElements()
            return elements

    class _getDevice(Interface.Method):
        name = 'getDevice'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = ValueObject.decode(rsp['_ret_'], agent)
            typecheck.is_valobj(_ret_, raritan.rpc.peripheral.Device, DecodeException)
            return _ret_

    class _assign(Interface.Method):
        name = 'assign'

        @staticmethod
        def encode(devid):
            typecheck.is_struct(devid, raritan.rpc.peripheral.DeviceID, AssertionError)
            args = {}
            args['devid'] = raritan.rpc.peripheral.DeviceID.encode(devid)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _assignAddress(Interface.Method):
        name = 'assignAddress'

        @staticmethod
        def encode(packageClass, address):
            typecheck.is_string(packageClass, AssertionError)
            typecheck.is_struct(address, raritan.rpc.peripheral.Address, AssertionError)
            args = {}
            args['packageClass'] = packageClass
            args['address'] = raritan.rpc.peripheral.Address.encode(address)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _unassign(Interface.Method):
        name = 'unassign'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getSettings(Interface.Method):
        name = 'getSettings'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.peripheral.DeviceSlot.Settings.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.peripheral.DeviceSlot.Settings, DecodeException)
            return _ret_

    class _setSettings(Interface.Method):
        name = 'setSettings'

        @staticmethod
        def encode(settings):
            typecheck.is_struct(settings, raritan.rpc.peripheral.DeviceSlot.Settings, AssertionError)
            args = {}
            args['settings'] = raritan.rpc.peripheral.DeviceSlot.Settings.encode(settings)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(DeviceSlot, self).__init__(target, agent)
        self.getDevice = DeviceSlot._getDevice(self)
        self.assign = DeviceSlot._assign(self)
        self.assignAddress = DeviceSlot._assignAddress(self)
        self.unassign = DeviceSlot._unassign(self)
        self.getSettings = DeviceSlot._getSettings(self)
        self.setSettings = DeviceSlot._setSettings(self)

#
# Section generated by IdlC from "PeripheralDevicePackage.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.peripheral


# structure
class PackageInfo(Structure):
    idlType = "peripheral.PackageInfo:2.0.1"
    elements = ["state", "position", "hwInfo", "fwInfo"]

    def __init__(self, state, position, hwInfo, fwInfo):
        typecheck.is_enum(state, raritan.rpc.peripheral.PackageInfo.State, AssertionError)
        for x0 in position:
            typecheck.is_struct(x0, raritan.rpc.peripheral.PosElement, AssertionError)
        typecheck.is_struct(hwInfo, raritan.rpc.peripheral.PackageInfo.HardwareInfo, AssertionError)
        typecheck.is_struct(fwInfo, raritan.rpc.peripheral.PackageInfo.FirmwareInfo, AssertionError)

        self.state = state
        self.position = position
        self.hwInfo = hwInfo
        self.fwInfo = fwInfo

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            state = raritan.rpc.peripheral.PackageInfo.State.decode(json['state']),
            position = [raritan.rpc.peripheral.PosElement.decode(x0, agent) for x0 in json['position']],
            hwInfo = raritan.rpc.peripheral.PackageInfo.HardwareInfo.decode(json['hwInfo'], agent),
            fwInfo = raritan.rpc.peripheral.PackageInfo.FirmwareInfo.decode(json['fwInfo'], agent),
        )
        return obj

    def encode(self):
        json = {}
        json['state'] = raritan.rpc.peripheral.PackageInfo.State.encode(self.state)
        json['position'] = [raritan.rpc.peripheral.PosElement.encode(x0) for x0 in self.position]
        json['hwInfo'] = raritan.rpc.peripheral.PackageInfo.HardwareInfo.encode(self.hwInfo)
        json['fwInfo'] = raritan.rpc.peripheral.PackageInfo.FirmwareInfo.encode(self.fwInfo)
        return json

    # enumeration
    class State(Enumeration):
        idlType = "peripheral.PackageInfo_2_0_1.State:1.0.0"
        values = ["NORMAL", "FW_UPDATE", "INTERNAL_ERROR", "CONFIG_ERROR"]

    State.NORMAL = State(0)
    State.FW_UPDATE = State(1)
    State.INTERNAL_ERROR = State(2)
    State.CONFIG_ERROR = State(3)

    # structure
    class HardwareInfo(Structure):
        idlType = "peripheral.PackageInfo_2_0_1.HardwareInfo:1.0.0"
        elements = ["serial", "packageClass", "model", "minDowngradeVersion", "revision"]

        def __init__(self, serial, packageClass, model, minDowngradeVersion, revision):
            typecheck.is_string(serial, AssertionError)
            typecheck.is_string(packageClass, AssertionError)
            typecheck.is_string(model, AssertionError)
            typecheck.is_int(minDowngradeVersion, AssertionError)
            typecheck.is_string(revision, AssertionError)

            self.serial = serial
            self.packageClass = packageClass
            self.model = model
            self.minDowngradeVersion = minDowngradeVersion
            self.revision = revision

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                serial = json['serial'],
                packageClass = json['packageClass'],
                model = json['model'],
                minDowngradeVersion = json['minDowngradeVersion'],
                revision = json['revision'],
            )
            return obj

        def encode(self):
            json = {}
            json['serial'] = self.serial
            json['packageClass'] = self.packageClass
            json['model'] = self.model
            json['minDowngradeVersion'] = self.minDowngradeVersion
            json['revision'] = self.revision
            return json

    # structure
    class FirmwareInfo(Structure):
        idlType = "peripheral.PackageInfo_2_0_1.FirmwareInfo:1.0.0"
        elements = ["compileDate", "version", "updateDate"]

        def __init__(self, compileDate, version, updateDate):
            typecheck.is_time(compileDate, AssertionError)
            typecheck.is_struct(version, raritan.rpc.peripheral.PackageInfo.FirmwareInfo.Version, AssertionError)
            typecheck.is_time(updateDate, AssertionError)

            self.compileDate = compileDate
            self.version = version
            self.updateDate = updateDate

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                compileDate = raritan.rpc.Time.decode(json['compileDate']),
                version = raritan.rpc.peripheral.PackageInfo.FirmwareInfo.Version.decode(json['version'], agent),
                updateDate = raritan.rpc.Time.decode(json['updateDate']),
            )
            return obj

        def encode(self):
            json = {}
            json['compileDate'] = raritan.rpc.Time.encode(self.compileDate)
            json['version'] = raritan.rpc.peripheral.PackageInfo.FirmwareInfo.Version.encode(self.version)
            json['updateDate'] = raritan.rpc.Time.encode(self.updateDate)
            return json

        # structure
        class Version(Structure):
            idlType = "peripheral.PackageInfo_2_0_1.FirmwareInfo.Version:1.0.0"
            elements = ["majorNumber", "minorNumber", "bootloaderVersion"]

            def __init__(self, majorNumber, minorNumber, bootloaderVersion):
                typecheck.is_int(majorNumber, AssertionError)
                typecheck.is_int(minorNumber, AssertionError)
                typecheck.is_int(bootloaderVersion, AssertionError)

                self.majorNumber = majorNumber
                self.minorNumber = minorNumber
                self.bootloaderVersion = bootloaderVersion

            @classmethod
            def decode(cls, json, agent):
                obj = cls(
                    majorNumber = json['majorNumber'],
                    minorNumber = json['minorNumber'],
                    bootloaderVersion = json['bootloaderVersion'],
                )
                return obj

            def encode(self):
                json = {}
                json['majorNumber'] = self.majorNumber
                json['minorNumber'] = self.minorNumber
                json['bootloaderVersion'] = self.bootloaderVersion
                return json

# interface
class Package(Interface):
    idlType = "peripheral.Package:1.0.1"

    class _getPackageInfo(Interface.Method):
        name = 'getPackageInfo'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.peripheral.PackageInfo.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.peripheral.PackageInfo, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(Package, self).__init__(target, agent)
        self.getPackageInfo = Package._getPackageInfo(self)

# interface
class DoorHandleControllerPackage(Package):
    idlType = "peripheral.DoorHandleControllerPackage:1.0.1"

    SUCCESS = 0

    ERR_NOT_SUPPORTED = 1

    ERR_OPERATION_FAILED = 2

    class _getSupportedHandleTypes(Interface.Method):
        name = 'getSupportedHandleTypes'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [x0 for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_string(x0, DecodeException)
            return _ret_

    class _getHandleType(Interface.Method):
        name = 'getHandleType'

        @staticmethod
        def encode(channel):
            typecheck.is_int(channel, AssertionError)
            args = {}
            args['channel'] = channel
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_string(_ret_, DecodeException)
            return _ret_

    class _setHandleType(Interface.Method):
        name = 'setHandleType'

        @staticmethod
        def encode(channel, handleType):
            typecheck.is_int(channel, AssertionError)
            typecheck.is_string(handleType, AssertionError)
            args = {}
            args['channel'] = channel
            args['handleType'] = handleType
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(DoorHandleControllerPackage, self).__init__(target, agent)
        self.getSupportedHandleTypes = DoorHandleControllerPackage._getSupportedHandleTypes(self)
        self.getHandleType = DoorHandleControllerPackage._getHandleType(self)
        self.setHandleType = DoorHandleControllerPackage._setHandleType(self)

#
# Section generated by IdlC from "PeripheralDeviceManager.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.event

import raritan.rpc.idl

import raritan.rpc.peripheral

import raritan.rpc.portsmodel

import raritan.rpc.sensors


# interface
class DeviceManager(Interface):
    idlType = "peripheral.DeviceManager:3.0.2"

    ERR_INVALID_PARAMS = 1

    # enumeration
    class ZCoordMode(Enumeration):
        idlType = "peripheral.DeviceManager_3_0_2.ZCoordMode:1.0.0"
        values = ["RACKUNITS", "FREEFORM"]

    ZCoordMode.RACKUNITS = ZCoordMode(0)
    ZCoordMode.FREEFORM = ZCoordMode(1)

    # structure
    class Settings(Structure):
        idlType = "peripheral.DeviceManager_3_0_2.Settings:1.0.0"
        elements = ["zCoordMode", "autoManageNewDevices", "deviceAltitude", "presenceDetectionTimeout", "defaultThresholdsMap", "maxActivePoweredDryContacts"]

        def __init__(self, zCoordMode, autoManageNewDevices, deviceAltitude, presenceDetectionTimeout, defaultThresholdsMap, maxActivePoweredDryContacts):
            typecheck.is_enum(zCoordMode, raritan.rpc.peripheral.DeviceManager.ZCoordMode, AssertionError)
            typecheck.is_bool(autoManageNewDevices, AssertionError)
            typecheck.is_float(deviceAltitude, AssertionError)
            typecheck.is_int(presenceDetectionTimeout, AssertionError)
            typecheck.is_int(maxActivePoweredDryContacts, AssertionError)

            self.zCoordMode = zCoordMode
            self.autoManageNewDevices = autoManageNewDevices
            self.deviceAltitude = deviceAltitude
            self.presenceDetectionTimeout = presenceDetectionTimeout
            self.defaultThresholdsMap = defaultThresholdsMap
            self.maxActivePoweredDryContacts = maxActivePoweredDryContacts

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                zCoordMode = raritan.rpc.peripheral.DeviceManager.ZCoordMode.decode(json['zCoordMode']),
                autoManageNewDevices = json['autoManageNewDevices'],
                deviceAltitude = json['deviceAltitude'],
                presenceDetectionTimeout = json['presenceDetectionTimeout'],
                defaultThresholdsMap = dict([(
                    elem['key'],
                    raritan.rpc.sensors.NumericSensor.Thresholds.decode(elem['value'], agent))
                    for elem in json['defaultThresholdsMap']]),
                maxActivePoweredDryContacts = json['maxActivePoweredDryContacts'],
            )
            return obj

        def encode(self):
            json = {}
            json['zCoordMode'] = raritan.rpc.peripheral.DeviceManager.ZCoordMode.encode(self.zCoordMode)
            json['autoManageNewDevices'] = self.autoManageNewDevices
            json['deviceAltitude'] = self.deviceAltitude
            json['presenceDetectionTimeout'] = self.presenceDetectionTimeout
            json['defaultThresholdsMap'] = [dict(
                key = k,
                value = raritan.rpc.sensors.NumericSensor.Thresholds.encode(v))
                for k, v in self.defaultThresholdsMap.items()]
            json['maxActivePoweredDryContacts'] = self.maxActivePoweredDryContacts
            return json

    # structure
    class MetaData(Structure):
        idlType = "peripheral.DeviceManager_3_0_2.MetaData:1.0.0"
        elements = ["oneWirePortCount", "onboardDeviceCount"]

        def __init__(self, oneWirePortCount, onboardDeviceCount):
            typecheck.is_int(oneWirePortCount, AssertionError)
            typecheck.is_int(onboardDeviceCount, AssertionError)

            self.oneWirePortCount = oneWirePortCount
            self.onboardDeviceCount = onboardDeviceCount

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                oneWirePortCount = json['oneWirePortCount'],
                onboardDeviceCount = json['onboardDeviceCount'],
            )
            return obj

        def encode(self):
            json = {}
            json['oneWirePortCount'] = self.oneWirePortCount
            json['onboardDeviceCount'] = self.onboardDeviceCount
            return json

    # structure
    class DeviceTypeInfo(Structure):
        idlType = "peripheral.DeviceManager_3_0_2.DeviceTypeInfo:1.0.0"
        elements = ["type", "isActuator", "identifier", "name", "defaultRange", "defaultDecDigits"]

        def __init__(self, type, isActuator, identifier, name, defaultRange, defaultDecDigits):
            typecheck.is_struct(type, raritan.rpc.sensors.Sensor.TypeSpec, AssertionError)
            typecheck.is_bool(isActuator, AssertionError)
            typecheck.is_string(identifier, AssertionError)
            typecheck.is_string(name, AssertionError)
            typecheck.is_struct(defaultRange, raritan.rpc.sensors.NumericSensor.Range, AssertionError)
            typecheck.is_int(defaultDecDigits, AssertionError)

            self.type = type
            self.isActuator = isActuator
            self.identifier = identifier
            self.name = name
            self.defaultRange = defaultRange
            self.defaultDecDigits = defaultDecDigits

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                type = raritan.rpc.sensors.Sensor.TypeSpec.decode(json['type'], agent),
                isActuator = json['isActuator'],
                identifier = json['identifier'],
                name = json['name'],
                defaultRange = raritan.rpc.sensors.NumericSensor.Range.decode(json['defaultRange'], agent),
                defaultDecDigits = json['defaultDecDigits'],
            )
            return obj

        def encode(self):
            json = {}
            json['type'] = raritan.rpc.sensors.Sensor.TypeSpec.encode(self.type)
            json['isActuator'] = self.isActuator
            json['identifier'] = self.identifier
            json['name'] = self.name
            json['defaultRange'] = raritan.rpc.sensors.NumericSensor.Range.encode(self.defaultRange)
            json['defaultDecDigits'] = self.defaultDecDigits
            return json

    # structure
    class FirmwareUpdateState(Structure):
        idlType = "peripheral.DeviceManager_3_0_2.FirmwareUpdateState:1.0.0"
        elements = ["active", "remaining"]

        def __init__(self, active, remaining):
            typecheck.is_bool(active, AssertionError)
            typecheck.is_int(remaining, AssertionError)

            self.active = active
            self.remaining = remaining

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                active = json['active'],
                remaining = json['remaining'],
            )
            return obj

        def encode(self):
            json = {}
            json['active'] = self.active
            json['remaining'] = self.remaining
            return json

    # value object
    class SettingsChangedEvent(raritan.rpc.event.UserEvent):
        idlType = "peripheral.DeviceManager_3_0_2.SettingsChangedEvent:1.0.0"

        def __init__(self, oldSettings, newSettings, actUserName, actIpAddr, source):
            super(raritan.rpc.peripheral.DeviceManager.SettingsChangedEvent, self).__init__(actUserName, actIpAddr, source)
            typecheck.is_struct(oldSettings, raritan.rpc.peripheral.DeviceManager.Settings, AssertionError)
            typecheck.is_struct(newSettings, raritan.rpc.peripheral.DeviceManager.Settings, AssertionError)

            self.oldSettings = oldSettings
            self.newSettings = newSettings

        def encode(self):
            json = super(raritan.rpc.peripheral.DeviceManager.SettingsChangedEvent, self).encode()
            json['oldSettings'] = raritan.rpc.peripheral.DeviceManager.Settings.encode(self.oldSettings)
            json['newSettings'] = raritan.rpc.peripheral.DeviceManager.Settings.encode(self.newSettings)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                oldSettings = raritan.rpc.peripheral.DeviceManager.Settings.decode(json['oldSettings'], agent),
                newSettings = raritan.rpc.peripheral.DeviceManager.Settings.decode(json['newSettings'], agent),
                # for event.UserEvent
                actUserName = json['actUserName'],
                actIpAddr = json['actIpAddr'],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["oldSettings", "newSettings"]
            elements = elements + super(raritan.rpc.peripheral.DeviceManager.SettingsChangedEvent, self).listElements()
            return elements

    # value object
    class DeviceEvent(raritan.rpc.idl.Event):
        idlType = "peripheral.DeviceManager_3_0_2.DeviceEvent:2.0.0"

        def __init__(self, devices, allDevices, source):
            super(raritan.rpc.peripheral.DeviceManager.DeviceEvent, self).__init__(source)
            for x0 in devices:
                typecheck.is_valobj(x0, raritan.rpc.peripheral.Device, AssertionError)
            for x0 in allDevices:
                typecheck.is_valobj(x0, raritan.rpc.peripheral.Device, AssertionError)

            self.devices = devices
            self.allDevices = allDevices

        def encode(self):
            json = super(raritan.rpc.peripheral.DeviceManager.DeviceEvent, self).encode()
            json['devices'] = [ValueObject.encode(x0) for x0 in self.devices]
            json['allDevices'] = [ValueObject.encode(x0) for x0 in self.allDevices]
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                devices = [ValueObject.decode(x0, agent) for x0 in json['devices']],
                allDevices = [ValueObject.decode(x0, agent) for x0 in json['allDevices']],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["devices", "allDevices"]
            elements = elements + super(raritan.rpc.peripheral.DeviceManager.DeviceEvent, self).listElements()
            return elements

    # value object
    class DeviceAddedEvent(DeviceEvent):
        idlType = "peripheral.DeviceManager_3_0_2.DeviceAddedEvent:2.0.0"

        def __init__(self, devices, allDevices, source):
            super(raritan.rpc.peripheral.DeviceManager.DeviceAddedEvent, self).__init__(devices, allDevices, source)

        def encode(self):
            json = super(raritan.rpc.peripheral.DeviceManager.DeviceAddedEvent, self).encode()
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                # for peripheral.DeviceManager_3_0_2.DeviceEvent_2_0_0
                devices = [ValueObject.decode(x0, agent) for x0 in json['devices']],
                allDevices = [ValueObject.decode(x0, agent) for x0 in json['allDevices']],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = []
            elements = elements + super(raritan.rpc.peripheral.DeviceManager.DeviceAddedEvent, self).listElements()
            return elements

    # value object
    class DeviceRemovedEvent(DeviceEvent):
        idlType = "peripheral.DeviceManager_3_0_2.DeviceRemovedEvent:2.0.0"

        def __init__(self, devices, allDevices, source):
            super(raritan.rpc.peripheral.DeviceManager.DeviceRemovedEvent, self).__init__(devices, allDevices, source)

        def encode(self):
            json = super(raritan.rpc.peripheral.DeviceManager.DeviceRemovedEvent, self).encode()
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                # for peripheral.DeviceManager_3_0_2.DeviceEvent_2_0_0
                devices = [ValueObject.decode(x0, agent) for x0 in json['devices']],
                allDevices = [ValueObject.decode(x0, agent) for x0 in json['allDevices']],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = []
            elements = elements + super(raritan.rpc.peripheral.DeviceManager.DeviceRemovedEvent, self).listElements()
            return elements

    # value object
    class UnknownDeviceAttachedEvent(raritan.rpc.idl.Event):
        idlType = "peripheral.DeviceManager_3_0_2.UnknownDeviceAttachedEvent:1.0.0"

        def __init__(self, romCode, position, source):
            super(raritan.rpc.peripheral.DeviceManager.UnknownDeviceAttachedEvent, self).__init__(source)
            typecheck.is_string(romCode, AssertionError)
            for x0 in position:
                typecheck.is_struct(x0, raritan.rpc.peripheral.PosElement, AssertionError)

            self.romCode = romCode
            self.position = position

        def encode(self):
            json = super(raritan.rpc.peripheral.DeviceManager.UnknownDeviceAttachedEvent, self).encode()
            json['romCode'] = self.romCode
            json['position'] = [raritan.rpc.peripheral.PosElement.encode(x0) for x0 in self.position]
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                romCode = json['romCode'],
                position = [raritan.rpc.peripheral.PosElement.decode(x0, agent) for x0 in json['position']],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["romCode", "position"]
            elements = elements + super(raritan.rpc.peripheral.DeviceManager.UnknownDeviceAttachedEvent, self).listElements()
            return elements

    # enumeration
    class DeviceFirmwareUpdateState(Enumeration):
        idlType = "peripheral.DeviceManager_3_0_2.DeviceFirmwareUpdateState:1.0.0"
        values = ["UPDATE_STARTED", "UPDATE_SUCCESSFUL", "UPDATE_FAILED"]

    DeviceFirmwareUpdateState.UPDATE_STARTED = DeviceFirmwareUpdateState(0)
    DeviceFirmwareUpdateState.UPDATE_SUCCESSFUL = DeviceFirmwareUpdateState(1)
    DeviceFirmwareUpdateState.UPDATE_FAILED = DeviceFirmwareUpdateState(2)

    # value object
    class DeviceFirmwareUpdateStateChangedEvent(raritan.rpc.idl.Event):
        idlType = "peripheral.DeviceManager_3_0_2.DeviceFirmwareUpdateStateChangedEvent:1.0.0"

        def __init__(self, oldVersion, newVersion, serial, state, source):
            super(raritan.rpc.peripheral.DeviceManager.DeviceFirmwareUpdateStateChangedEvent, self).__init__(source)
            typecheck.is_string(oldVersion, AssertionError)
            typecheck.is_string(newVersion, AssertionError)
            typecheck.is_string(serial, AssertionError)
            typecheck.is_enum(state, raritan.rpc.peripheral.DeviceManager.DeviceFirmwareUpdateState, AssertionError)

            self.oldVersion = oldVersion
            self.newVersion = newVersion
            self.serial = serial
            self.state = state

        def encode(self):
            json = super(raritan.rpc.peripheral.DeviceManager.DeviceFirmwareUpdateStateChangedEvent, self).encode()
            json['oldVersion'] = self.oldVersion
            json['newVersion'] = self.newVersion
            json['serial'] = self.serial
            json['state'] = raritan.rpc.peripheral.DeviceManager.DeviceFirmwareUpdateState.encode(self.state)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                oldVersion = json['oldVersion'],
                newVersion = json['newVersion'],
                serial = json['serial'],
                state = raritan.rpc.peripheral.DeviceManager.DeviceFirmwareUpdateState.decode(json['state']),
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["oldVersion", "newVersion", "serial", "state"]
            elements = elements + super(raritan.rpc.peripheral.DeviceManager.DeviceFirmwareUpdateStateChangedEvent, self).listElements()
            return elements

    # value object
    class FirmwareUpdateStateChangedEvent(raritan.rpc.idl.Event):
        idlType = "peripheral.DeviceManager_3_0_2.FirmwareUpdateStateChangedEvent:1.0.0"

        def __init__(self, newState, source):
            super(raritan.rpc.peripheral.DeviceManager.FirmwareUpdateStateChangedEvent, self).__init__(source)
            typecheck.is_struct(newState, raritan.rpc.peripheral.DeviceManager.FirmwareUpdateState, AssertionError)

            self.newState = newState

        def encode(self):
            json = super(raritan.rpc.peripheral.DeviceManager.FirmwareUpdateStateChangedEvent, self).encode()
            json['newState'] = raritan.rpc.peripheral.DeviceManager.FirmwareUpdateState.encode(self.newState)
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                newState = raritan.rpc.peripheral.DeviceManager.FirmwareUpdateState.decode(json['newState'], agent),
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["newState"]
            elements = elements + super(raritan.rpc.peripheral.DeviceManager.FirmwareUpdateStateChangedEvent, self).listElements()
            return elements

    # value object
    class PackageEvent(raritan.rpc.idl.Event):
        idlType = "peripheral.DeviceManager_3_0_2.PackageEvent:2.0.0"

        def __init__(self, packageInfos, allPackages, source):
            super(raritan.rpc.peripheral.DeviceManager.PackageEvent, self).__init__(source)
            for x0 in packageInfos:
                typecheck.is_struct(x0, raritan.rpc.peripheral.PackageInfo, AssertionError)
            for x0 in allPackages:
                typecheck.is_struct(x0, raritan.rpc.peripheral.PackageInfo, AssertionError)

            self.packageInfos = packageInfos
            self.allPackages = allPackages

        def encode(self):
            json = super(raritan.rpc.peripheral.DeviceManager.PackageEvent, self).encode()
            json['packageInfos'] = [raritan.rpc.peripheral.PackageInfo.encode(x0) for x0 in self.packageInfos]
            json['allPackages'] = [raritan.rpc.peripheral.PackageInfo.encode(x0) for x0 in self.allPackages]
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                packageInfos = [raritan.rpc.peripheral.PackageInfo.decode(x0, agent) for x0 in json['packageInfos']],
                allPackages = [raritan.rpc.peripheral.PackageInfo.decode(x0, agent) for x0 in json['allPackages']],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = ["packageInfos", "allPackages"]
            elements = elements + super(raritan.rpc.peripheral.DeviceManager.PackageEvent, self).listElements()
            return elements

    # value object
    class PackageAddedEvent(PackageEvent):
        idlType = "peripheral.DeviceManager_3_0_2.PackageAddedEvent:2.0.0"

        def __init__(self, packageInfos, allPackages, source):
            super(raritan.rpc.peripheral.DeviceManager.PackageAddedEvent, self).__init__(packageInfos, allPackages, source)

        def encode(self):
            json = super(raritan.rpc.peripheral.DeviceManager.PackageAddedEvent, self).encode()
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                # for peripheral.DeviceManager_3_0_2.PackageEvent_2_0_0
                packageInfos = [raritan.rpc.peripheral.PackageInfo.decode(x0, agent) for x0 in json['packageInfos']],
                allPackages = [raritan.rpc.peripheral.PackageInfo.decode(x0, agent) for x0 in json['allPackages']],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = []
            elements = elements + super(raritan.rpc.peripheral.DeviceManager.PackageAddedEvent, self).listElements()
            return elements

    # value object
    class PackageRemovedEvent(PackageEvent):
        idlType = "peripheral.DeviceManager_3_0_2.PackageRemovedEvent:2.0.0"

        def __init__(self, packageInfos, allPackages, source):
            super(raritan.rpc.peripheral.DeviceManager.PackageRemovedEvent, self).__init__(packageInfos, allPackages, source)

        def encode(self):
            json = super(raritan.rpc.peripheral.DeviceManager.PackageRemovedEvent, self).encode()
            return json

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                # for peripheral.DeviceManager_3_0_2.PackageEvent_2_0_0
                packageInfos = [raritan.rpc.peripheral.PackageInfo.decode(x0, agent) for x0 in json['packageInfos']],
                allPackages = [raritan.rpc.peripheral.PackageInfo.decode(x0, agent) for x0 in json['allPackages']],
                # for idl.Event
                source = Interface.decode(json['source'], agent),
            )
            return obj

        def listElements(self):
            elements = []
            elements = elements + super(raritan.rpc.peripheral.DeviceManager.PackageRemovedEvent, self).listElements()
            return elements

    # structure
    class Statistics(Structure):
        idlType = "peripheral.DeviceManager_3_0_2.Statistics:1.0.0"
        elements = ["cSumErrCnt"]

        def __init__(self, cSumErrCnt):
            typecheck.is_int(cSumErrCnt, AssertionError)

            self.cSumErrCnt = cSumErrCnt

        @classmethod
        def decode(cls, json, agent):
            obj = cls(
                cSumErrCnt = json['cSumErrCnt'],
            )
            return obj

        def encode(self):
            json = {}
            json['cSumErrCnt'] = self.cSumErrCnt
            return json

    class _getDeviceSlots(Interface.Method):
        name = 'getDeviceSlots'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [Interface.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_interface(x0, raritan.rpc.peripheral.DeviceSlot, DecodeException)
            return _ret_

    class _getDeviceSlot(Interface.Method):
        name = 'getDeviceSlot'

        @staticmethod
        def encode(idx):
            typecheck.is_int(idx, AssertionError)
            args = {}
            args['idx'] = idx
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = Interface.decode(rsp['_ret_'], agent)
            typecheck.is_interface(_ret_, raritan.rpc.peripheral.DeviceSlot, DecodeException)
            return _ret_

    class _getDiscoveredDevices(Interface.Method):
        name = 'getDiscoveredDevices'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [ValueObject.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_valobj(x0, raritan.rpc.peripheral.Device, DecodeException)
            return _ret_

    class _getDiscoveredPackageInfos(Interface.Method):
        name = 'getDiscoveredPackageInfos'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.peripheral.PackageInfo.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.peripheral.PackageInfo, DecodeException)
            return _ret_

    class _getSettings(Interface.Method):
        name = 'getSettings'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.peripheral.DeviceManager.Settings.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.peripheral.DeviceManager.Settings, DecodeException)
            return _ret_

    class _setSettings(Interface.Method):
        name = 'setSettings'

        @staticmethod
        def encode(settings):
            typecheck.is_struct(settings, raritan.rpc.peripheral.DeviceManager.Settings, AssertionError)
            args = {}
            args['settings'] = raritan.rpc.peripheral.DeviceManager.Settings.encode(settings)
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_

    class _getMetaData(Interface.Method):
        name = 'getMetaData'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.peripheral.DeviceManager.MetaData.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.peripheral.DeviceManager.MetaData, DecodeException)
            return _ret_

    class _getDeviceTypeInfos(Interface.Method):
        name = 'getDeviceTypeInfos'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [raritan.rpc.peripheral.DeviceManager.DeviceTypeInfo.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_struct(x0, raritan.rpc.peripheral.DeviceManager.DeviceTypeInfo, DecodeException)
            return _ret_

    class _getFirmwareUpdateState(Interface.Method):
        name = 'getFirmwareUpdateState'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.peripheral.DeviceManager.FirmwareUpdateState.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.peripheral.DeviceManager.FirmwareUpdateState, DecodeException)
            return _ret_

    class _getStatistics(Interface.Method):
        name = 'getStatistics'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = raritan.rpc.peripheral.DeviceManager.Statistics.decode(rsp['_ret_'], agent)
            typecheck.is_struct(_ret_, raritan.rpc.peripheral.DeviceManager.Statistics, DecodeException)
            return _ret_

    class _getDiscoveredPackages(Interface.Method):
        name = 'getDiscoveredPackages'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = [Interface.decode(x0, agent) for x0 in rsp['_ret_']]
            for x0 in _ret_:
                typecheck.is_interface(x0, raritan.rpc.peripheral.Package, DecodeException)
            return _ret_

    class _getPortFuse(Interface.Method):
        name = 'getPortFuse'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = Interface.decode(rsp['_ret_'], agent)
            typecheck.is_interface(_ret_, raritan.rpc.portsmodel.PortFuse, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(DeviceManager, self).__init__(target, agent)
        self.getDeviceSlots = DeviceManager._getDeviceSlots(self)
        self.getDeviceSlot = DeviceManager._getDeviceSlot(self)
        self.getDiscoveredDevices = DeviceManager._getDiscoveredDevices(self)
        self.getDiscoveredPackageInfos = DeviceManager._getDiscoveredPackageInfos(self)
        self.getSettings = DeviceManager._getSettings(self)
        self.setSettings = DeviceManager._setSettings(self)
        self.getMetaData = DeviceManager._getMetaData(self)
        self.getDeviceTypeInfos = DeviceManager._getDeviceTypeInfos(self)
        self.getFirmwareUpdateState = DeviceManager._getFirmwareUpdateState(self)
        self.getStatistics = DeviceManager._getStatistics(self)
        self.getDiscoveredPackages = DeviceManager._getDiscoveredPackages(self)
        self.getPortFuse = DeviceManager._getPortFuse(self)

#
# Section generated by IdlC from "PeripheralG2Production.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException

# interface
class G2Production(Interface):
    idlType = "peripheral.G2Production:4.0.0"

    SUCCESS = 0

    ERR_INVALID_PARAMS = 1

    ERR_NO_CONFIG_MODE = 2

    ERR_NO_DEVICE = 3

    ERR_OPERATION_FAILED = 4

    class _programSerialNumber(Interface.Method):
        name = 'programSerialNumber'

        @staticmethod
        def encode(romcode, serial):
            typecheck.is_string(romcode, AssertionError)
            typecheck.is_string(serial, AssertionError)
            args = {}
            args['romcode'] = romcode
            args['serial'] = serial
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_int(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(G2Production, self).__init__(target, agent)
        self.programSerialNumber = G2Production._programSerialNumber(self)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "LhxSupport.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException

# interface
class Support(Interface):
    idlType = "lhx.Support:1.0.0"

    class _setEnabled(Interface.Method):
        name = 'setEnabled'

        @staticmethod
        def encode(enabled):
            typecheck.is_bool(enabled, AssertionError)
            args = {}
            args['enabled'] = enabled
            return args

        @staticmethod
        def decode(rsp, agent):
            return None

    class _isEnabled(Interface.Method):
        name = 'isEnabled'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            _ret_ = rsp['_ret_']
            typecheck.is_bool(_ret_, DecodeException)
            return _ret_
    def __init__(self, target, agent):
        super(Support, self).__init__(target, agent)
        self.setEnabled = Support._setEnabled(self)
        self.isEnabled = Support._isEnabled(self)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "ResMon.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.res_mon


# structure
class Entry(Structure):
    idlType = "res_mon.Entry:1.0.0"
    elements = ["type", "name", "value"]

    def __init__(self, type, name, value):
        typecheck.is_enum(type, raritan.rpc.res_mon.Entry.Type, AssertionError)
        typecheck.is_string(name, AssertionError)
        typecheck.is_long(value, AssertionError)

        self.type = type
        self.name = name
        self.value = value

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            type = raritan.rpc.res_mon.Entry.Type.decode(json['type']),
            name = json['name'],
            value = int(json['value']),
        )
        return obj

    def encode(self):
        json = {}
        json['type'] = raritan.rpc.res_mon.Entry.Type.encode(self.type)
        json['name'] = self.name
        json['value'] = self.value
        return json

    # enumeration
    class Type(Enumeration):
        idlType = "res_mon.Entry.Type:1.0.0"
        values = ["GLOBAL_CPU_USAGE", "GLOBAL_FREE_MEM", "GLOBAL_PROC_COUNT", "FS_FREE_SPACE", "FS_FREE_INODES", "PROC_CPU_USAGE", "PROC_VM_SIZE", "PROC_FREE_FILE_DESC", "PROC_LIFE_TIME", "PROC_COUNT"]

    Type.GLOBAL_CPU_USAGE = Type(0)
    Type.GLOBAL_FREE_MEM = Type(1)
    Type.GLOBAL_PROC_COUNT = Type(2)
    Type.FS_FREE_SPACE = Type(3)
    Type.FS_FREE_INODES = Type(4)
    Type.PROC_CPU_USAGE = Type(5)
    Type.PROC_VM_SIZE = Type(6)
    Type.PROC_FREE_FILE_DESC = Type(7)
    Type.PROC_LIFE_TIME = Type(8)
    Type.PROC_COUNT = Type(9)

# interface
class ResMon(Interface):
    idlType = "res_mon.ResMon:1.0.0"

    class _getDataEntries(Interface.Method):
        name = 'getDataEntries'

        @staticmethod
        def encode():
            args = {}
            return args

        @staticmethod
        def decode(rsp, agent):
            entries = [raritan.rpc.res_mon.Entry.decode(x0, agent) for x0 in rsp['entries']]
            for x0 in entries:
                typecheck.is_struct(x0, raritan.rpc.res_mon.Entry, DecodeException)
            return entries
    def __init__(self, target, agent):
        super(ResMon, self).__init__(target, agent)
        self.getDataEntries = ResMon._getDataEntries(self)



# SPDX-License-Identifier: BSD-3-Clause
#
# Copyright 2020 Raritan Inc. All rights reserved.
#
# This is an auto-generated file.

#
# Section generated by IdlC from "RadiusServerSettings.idl"
#

import raritan.rpc
from raritan.rpc import Interface, Structure, ValueObject, Enumeration, typecheck, DecodeException
import raritan.rpc.radius


# enumeration
class AuthType(Enumeration):
    idlType = "radius.AuthType:2.0.0"
    values = ["PAP", "CHAP", "MSCHAPv2"]

AuthType.PAP = AuthType(0)
AuthType.CHAP = AuthType(1)
AuthType.MSCHAPv2 = AuthType(2)

# structure
class ServerSettings(Structure):
    idlType = "radius.ServerSettings:2.0.0"
    elements = ["id", "server", "sharedSecret", "udpAuthPort", "udpAccountPort", "timeout", "retries", "authType"]

    def __init__(self, id, server, sharedSecret, udpAuthPort, udpAccountPort, timeout, retries, authType):
        typecheck.is_string(id, AssertionError)
        typecheck.is_string(server, AssertionError)
        typecheck.is_string(sharedSecret, AssertionError)
        typecheck.is_int(udpAuthPort, AssertionError)
        typecheck.is_int(udpAccountPort, AssertionError)
        typecheck.is_int(timeout, AssertionError)
        typecheck.is_int(retries, AssertionError)
        typecheck.is_enum(authType, raritan.rpc.radius.AuthType, AssertionError)

        self.id = id
        self.server = server
        self.sharedSecret = sharedSecret
        self.udpAuthPort = udpAuthPort
        self.udpAccountPort = udpAccountPort
        self.timeout = timeout
        self.retries = retries
        self.authType = authType

    @classmethod
    def decode(cls, json, agent):
        obj = cls(
            id = json['id'],
            server = json['server'],
            sharedSecret = json['sharedSecret'],
            udpAuthPort = json['udpAuthPort'],
            udpAccountPort = json['udpAccountPort'],
            timeout = json['timeout'],
            retries = json['retries'],
            authType = raritan.rpc.radius.AuthType.decode(json['authType']),
        )
        return obj

    def encode(self):
        json = {}
        json['id'] = self.id
        json['server'] = self.server
        json['sharedSecret'] = self.sharedSecret
        json['udpAuthPort'] = self.udpAuthPort
        json['udpAccountPort'] = self.udpAccountPort
        json['timeout'] = self.timeout
        json['retries'] = self.retries
        json['authType'] = raritan.rpc.radius.AuthType.encode(self.authType)
        return json
